Package: GOaGO
Title: Gene Ontology enrichment analysis of gene pairs
Version: 1.0.0
Authors@R: person("Aleksander", "Jankowski",
    email = "aleksander.jankowski@uw.edu.pl",
    role = c("aut", "cre"),
    comment = c(ORCID = "0000-0002-2212-6224"))
Description: GO-a-GO annotates Gene Ontology terms that are enriched in a given
    set of gene pairs. The enrichment is calculated from a permutation test for
    overrepresentation of gene pairs that are associated with a shared term.
    Such gene pairs are counted for the original set of gene pairs and compared
    against randomized sets in which the structure of the pairs is preserved,
    but the gene identities (including the associated terms) are permuted.
License: Artistic-2.0
Encoding: UTF-8
LazyData: false
Imports: AnnotationDbi, BiocGenerics, BiocParallel, clusterProfiler,
        data.table, DOSE, GenomeInfoDb, GenomicRanges, ggplot2,
        ggridges, Matrix, qvalue, S4Vectors
Depends: R (>= 4.4.0), methods
biocViews: GO, GeneSetEnrichment
URL: https://github.com/ajank/GOaGO
BugReports: https://github.com/ajank/GOaGO/issues
RoxygenNote: 7.3.3
Suggests: BiocStyle, GenomicInteractions, ggrepel, knitr, org.Hs.eg.db,
        rmarkdown, rtracklayer, testthat,
        TxDb.Hsapiens.UCSC.hg19.knownGene
Config/testthat/edition: 3
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/GOaGO
git_branch: RELEASE_3_23
git_last_commit: ace0eb8
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:33:02 UTC; biocbuild
Author: Aleksander Jankowski [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2212-6224>)
Maintainer: Aleksander Jankowski <aleksander.jankowski@uw.edu.pl>

export("GOaGO")
export("annotateAnchors")
export("annotateInteractions")
export("convertTranscriptsToTSS")
export("DotPlot")
export("genePairs")
export("keyType")
export("RidgePlot")
export("termGenePairs")

exportClasses("GOaGO-result")
exportMethods("as.data.frame")
S3method("as.data.frame", "GOaGO-result")
exportMethods("organism")
exportMethods("show")

importFrom("AnnotationDbi", "mapIds", "species")
importFrom("BiocGenerics", "as.data.frame", "end", "organism", "start",
    "strand", "width")
importFrom("BiocParallel", "bplapply")
importFrom("clusterProfiler", "enrichGO")
importFrom("data.table", ".I", ":=", "as.data.table", "data.table", "dcast",
    "setcolorder", "setkey", "setnafill")
importFrom("DOSE", "theme_dose")
importFrom("GenomeInfoDb", "seqinfo", "seqnames")
importFrom("GenomicRanges", "distance", "GRanges", "nearest", "resize")
importFrom("ggplot2", ".data", "aes", "element_text", "expansion", "geom_point",
    "ggplot", "label_wrap_gen", "labs", "scale_color_manual",
    "scale_fill_manual", "scale_size", "scale_size_area", "scale_x_continuous",
    "scale_y_discrete", "theme")
importFrom("ggridges", "geom_density_ridges")
importFrom("Matrix", "Matrix", "colSums", "which")
importFrom("methods", "new", "show")
importFrom("qvalue", "qvalue")
importFrom("S4Vectors", "mcols", "mcols<-", "metadata", "queryHits",
    "subjectHits")
importFrom("stats", "p.adjust")
importFrom("utils", "head", "str")

<!-- README.md is generated from README.Rmd. Please edit that file -->

# GO-a-GO

<!-- badges: start -->

<!-- badges: end -->

GO-a-GO annotates Gene Ontology terms that are enriched in a given set
of gene pairs. The enrichment is calculated from a permutation test for
overrepresentation of gene pairs that are associated with a shared term.
Such gene pairs are counted for the original set of gene pairs and
compared against randomized sets in which the structure of the pairs is
preserved, but the gene identities (including the associated terms) are
permuted.

## Installation

You can install the development version of GO-a-GO from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("ajank/GOaGO")
```

## Example

Let us run GO-a-GO on a set of gene pairs associated with chromatin
loops identified in human cell line GM12878. We can inspect a few rows
of the dataset:

``` r
library(GOaGO)
#> 

data("genePairsGM12878")
head(genePairsGM12878)
#>    interactionID chrom1    start1      end1 geneID1      tss1 strand1 chrom2
#>            <int> <char>     <int>     <int>  <char>     <int>  <char> <char>
#> 1:             3  chr10 101190001 101195000    2805 101190530       -  chr10
#> 2:             7  chr10 102100001 102110000    6319 102106772       +  chr10
#> 3:             7  chr10 102100001 102110000    6319 102106772       +  chr10
#> 4:            10  chr10 102810001 102815000   81621 102820999       +  chr10
#> 5:            19  chr10 103600001 103605000   30819 103603677       -  chr10
#> 6:            23  chr10 103985001 103990000   83401 103986143       +  chr10
#>       start2      end2 geneID2      tss2 strand2
#>        <int>     <int>  <char>     <int>  <char>
#> 1: 101370001 101375000   81894 101372701       -
#> 2: 102270001 102280000   25956 102276754       -
#> 3: 102270001 102280000   25956 102279595       -
#> 4: 102900001 102905000    3195 102891061       +
#> 5: 103830001 103835000   79803 103825124       +
#> 6: 104160001 104165000    5662 104169041       -
```

The column names ending with `1` and `2` refer to the first and second
loop anchors, respectively. The essential columns are: `geneID1` and
`geneID2` (gene identifiers from a database of choice) and
`interactionID` (identifier of a chromatin loop). As some loop anchors
overlapped multiple Transcription Start Sites, possibly of many genes,
for these loops the dataset contains all combinations.

When running GO-a-GO, we should specify that gene identifiers are from
the Entrez database, and use the Bioconductor `org.Hs.eg.db` package as
the source of Gene Ontology annotations for human genes, taking all
three subontologies (Biological Process, Molecular Function, and
Cellular Component). We will run the default 10,000 permutations, and
also use the default *p*-value cutoff of 0.05 with Benjamini-Hochberg
correction:

``` r
library(org.Hs.eg.db)

# set the number of CPU threads to use
library(BiocParallel)
options(MulticoreParam = MulticoreParam(workers = 2))

goago <- GOaGO(genePairsGM12878,
    keyType = "ENTREZID", OrgDb = org.Hs.eg.db, ont = "ALL"
)
#> Warning in uniqueGenePairs(genePairs): removing 509 duplicated gene pair(s)
#> Warning in uniqueGenePairs(genePairs): removing 87 gene pair(s) containing the
#> same gene twice
```

We can inspect the enriched GO terms:

``` r
head(as.data.frame(goago))
#>   ONTOLOGY         ID
#> 1       BP GO:0034728
#> 2       BP GO:0006334
#> 3       BP GO:0071824
#> 4       BP GO:0040029
#> 5       BP GO:0002495
#> 6       BP GO:0019886
#>                                                                         Description
#> 1                                                           nucleosome organization
#> 2                                                               nucleosome assembly
#> 3                                                  protein-DNA complex organization
#> 4                                          epigenetic regulation of gene expression
#> 5           antigen processing and presentation of peptide antigen via MHC class II
#> 6 antigen processing and presentation of exogenous peptide antigen via MHC class II
#>   Count   PairRatio      BgRatio FoldEnrichment pvalue p.adjust qvalue
#> 1    19 0.010900746 1.580608e-04       68.96552      0        0      0
#> 2    19 0.010900746 1.053356e-04      103.48584      0        0      0
#> 3    19 0.010900746 3.423982e-04       31.83646      0        0      0
#> 4    15 0.008605852 4.303500e-04       19.99733      0        0      0
#> 5     3 0.001721170 1.491681e-05      115.38462      0        0      0
#> 6     3 0.001721170 1.187608e-05      144.92754      0        0      0
```

Note that some of the *p*-values have the value of zero, which means
that for all the randomizations fewer gene pairs were associated with a
given term than in the input dataset.

We can plot the enriched GO terms as a dotplot. The X-axis shows the
fold enrichment of the fraction of gene pairs sharing the given term,
calculated as a quotient of `PairRatio` (the fraction in the input gene
pairs) and `BgRatio` (the fraction in permuted ones). Point size
indicates the number of input gene pairs sharing the given term, and
point color – the adjusted *p*-value:

``` r
DotPlot(goago)
```

<img src="man/figures/README-dotplot-1.png" width="100%" />

Note that by default only the terms associated with at least 5 gene
pairs are shown; you can change this by setting `minCount` to any other
value.

We can also see the sampling distributions of numbers of gene pairs
sharing each GO term, obtained for the randomized gene pairs. From these
distributions, empirical *p*-values were calculated:

``` r
RidgePlot(goago)
#> Picking joint bandwidth of 0.0684
```

<img src="man/figures/README-ridgeplot-1.png" width="100%" />

##' @name show
##' @aliases show,GOaGO-result-method
##' @rdname GOaGO-accessors
##' @usage show(object)
##' @export
setMethod(
    "show", signature(object = "GOaGO-result"),
    function(object) {
        cat("#\n")
        cat("# Gene Ontology over-representation test in a set of gene pairs\n")

        cat("#\n")
        cat("#...@organism", "\t", object@organism, "\n")
        cat("#...@ontology", "\t", object@ontology, "\n")
        cat("#...@keyType", "\t", object@keyType, "\n")

        cat("#\n")
        cat(sprintf(
            "#...%d gene pairs considered, %d permutations\n",
            nrow(object@genePairs), object@numPermutations
        ))
        cat(sprintf(
            "#...identified GO terms shared by at least %d %s\n",
            object@minCount,
            ifelse(object@minCount == 1, "gene pair", "gene pairs")
        ))
        cat(sprintf(
            "#...p-values adjusted by '%s' with cutoff < %s\n",
            object@pAdjustMethod, object@pvalueCutoff
        ))

        cat("#\n")
        n <- nrow(object@result)
        cat(sprintf(
            "#...%d enriched %s found\n", n,
            ifelse(n == 1, "term", "terms")
        ))
        if (n > 0) str(object@result)
    }
)

# Internal function to select only the terms that are associated to at least
# `minCount` gene pairs, and convert the terms (both `ID` and `Description`
# columns) to factors ordered by the column `orderBy`. Finally, select only the
# number of terms (or the terms themselves) specified by `showCategory`.
.sortedResult <- function(
    object,
    minCount = Inf,
    showCategory = Inf,
    orderBy = "FoldEnrichment",
    decreasing = TRUE
) {
    # prevent "no visible binding for global variable" NOTEs in R CMD check
    Count <- ID <- Description <- NULL

    dt <- as.data.table(object)
    dt <- dt[Count >= minCount, ]

    if (!orderBy %in% colnames(dt)) {
        message("wrong orderBy argument: ", orderBy)
    }

    dt <- dt[order(dt[[orderBy]], decreasing = decreasing), ]

    if (is.numeric(showCategory)) {
        dt <- head(dt, showCategory)
    } else {
        dt <- dt[ID %in% showCategory, ]
    }

    # note the reverse order of factors, so that the terms show up as expected
    # (top-to-bottom, reverse to the scale direction) in the final plot
    dt[, ID := factor(ID, rev(ID))]
    dt[, Description := factor(Description, rev(Description))]

    return(dt)
}


# Internal function to wrap labels at a given wrap length, or use a labeller
# function provided.
.label_format <- function(label_format = 50) {
    if (is.function(label_format)) {
        label_func <- label_format
    } else {
        label_func <- label_wrap_gen(label_format)
    }
    return(label_func)
}


##' Dotplot of the enriched Gene Ontology terms
##'
##' By default, plots the most enriched terms, with fold enrichment on the
##' X-axis, point size indicating the number of gene pairs sharing the given
##' term, and point color -- the adjusted p-value.
##'
##' @param object GO-a-GO results of class \code{GOaGO-result}
##' @param minCount plot only the GO terms that are associated to at least the
##'   given number of gene pairs
##' @param x Variable for X-axis, one of \code{"FoldEnrichment"},
##'   \code{"PairRatio"} and \code{"Count"}.
##' @param color Variable used to color enriched terms, e.g. \code{"pvalue"},
##'   \code{"p.adjust"} or \code{"qvalue"}.
##' @param size Variable used to scale the sizes of points, one of
##'   \code{"FoldEnrichment"}, \code{"PairRatio"} and \code{"Count"}.
##' @param showCategory number of terms to display or a vector of terms
##' @param orderBy The order of the Y-axis, one of \code{"FoldEnrichment"},
##'   \code{"PairRatio"} and \code{"Count"}.
##' @param decreasing logical. Should the \code{orderBy} order be increasing or
##'   decreasing?
##' @param font.size font size
##' @param label_format a numeric value sets wrap length, alternatively a custom
##'   function to format axis labels. By default wraps names longer than 50
##'   characters.
##' @returns A \code{ggplot} object that can be further customized using the
##'   \code{ggplot2} package.
##' @export
##' @examples
##' library(org.Hs.eg.db)
##' data("genePairsGM12878")
##'
##' goago <- GOaGO(genePairsGM12878, keyType = "ENTREZID", OrgDb = org.Hs.eg.db)
##' DotPlot(goago)
DotPlot <- function(
    object,
    minCount = 5,
    x = "FoldEnrichment",
    color = "p.adjust",
    size = "Count",
    showCategory = 10,
    orderBy = "FoldEnrichment",
    decreasing = TRUE,
    font.size = 12,
    label_format = 50
) {
    # prevent "no visible binding for global variable" NOTEs in R CMD check
    Description <- NULL

    dt <- .sortedResult(object,
        minCount = minCount, showCategory = showCategory, orderBy = orderBy,
        decreasing = decreasing
    )

    label_func <- .label_format(label_format)

    p <- ggplot(
        dt, aes(
            x = .data[[x]], y = Description, size = .data[[size]],
            fill = .data[[color]]
        )
    ) +
        geom_point(shape = 21) +
        labs(y = NULL) +
        scale_x_continuous(expand = expansion(mult = 0.1)) +
        scale_y_discrete(labels = label_func) +
        theme_dose(font.size)

    if (size == "Count") {
        # ensure that a value of 0 is mapped to a size of 0
        p <- p + scale_size_area(max_size = 8)
    } else {
        p <- p + scale_size(range = c(3, 8))
    }

    return(p)
}


##' Ridgeplot of the sampling distributions for the randomized gene pairs
##'
##' Ridgeplot of the sampling distributions of numbers of gene pairs sharing
##' each enriched Gene Ontology term, obtained for the randomized gene pairs.
##'
##' @param object GO-a-GO results of class \code{GOaGO-result}
##' @param minCount plot only the GO terms that are associated to at least the
##'   given number of gene pairs
##' @param showCategory number of terms to display or a vector of terms
##' @param orderBy The order of the Y-axis, one of \code{"FoldEnrichment"},
##'   \code{"PairRatio"} and \code{"Count"}.
##' @param decreasing logical. Should the \code{orderBy} order be increasing or
##'   decreasing?
##' @param font.size font size
##' @param label_format a numeric value sets wrap length, alternatively a custom
##'   function to format axis labels. By default wraps names longer than 50
##'   characters.
##' @returns A \code{ggplot} object that can be further customized using the
##'   \code{ggplot2} package.
##' @export
##' @examples
##' library(org.Hs.eg.db)
##' data("genePairsGM12878")
##'
##' goago <- GOaGO(genePairsGM12878, keyType = "ENTREZID", OrgDb = org.Hs.eg.db)
##' RidgePlot(goago)
RidgePlot <- function(
    object,
    minCount = 5,
    showCategory = 10,
    orderBy = "FoldEnrichment",
    decreasing = TRUE,
    font.size = 12,
    label_format = 50
) {
    # prevent "no visible binding for global variable" NOTEs in R CMD check
    Count <- Description <- NULL

    dt <- .sortedResult(object,
        minCount = minCount, showCategory = showCategory, orderBy = orderBy,
        decreasing = decreasing
    )

    dt_permuted <- merge(dt[, c("ID", "Description")],
        object@permutedResult,
        by = "ID"
    )

    label_func <- .label_format(label_format)

    col_color <- c(`TRUE` = "#e41a1c", `FALSE` = "black")
    col_fill <- c(`TRUE` = NA, `FALSE` = "gray70")
    lab <- c(`TRUE` = "actual gene pairs", `FALSE` = "randomized gene pairs")

    p <- ggplot(
        dt_permuted,
        aes(x = Count, y = Description, color = FALSE, fill = FALSE)
    ) +
        geom_density_ridges() +
        geom_point(data = dt, aes(x = Count, color = TRUE, fill = TRUE)) +
        scale_color_manual(NULL, values = col_color, labels = lab) +
        scale_fill_manual(NULL, values = col_fill, labels = lab) +
        labs(x = "Number of gene pairs sharing a term", y = NULL) +
        scale_y_discrete(labels = label_func) +
        theme_dose(font.size) +
        theme(legend.position = "bottom")

    return(p)
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "##"
)

## ----install, eval = FALSE----------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE)) {
#     install.packages("BiocManager")
# }
# 
# BiocManager::install("GOaGO")

## ----start, message = FALSE---------------------------------------------------
library(GOaGO)

## ----dataset------------------------------------------------------------------
data("genePairsGM12878")
head(genePairsGM12878)

## ----nrow---------------------------------------------------------------------
nrow(genePairsGM12878)

## ----goago, message = FALSE---------------------------------------------------
library(org.Hs.eg.db)

# set the number of CPU threads to use
library(BiocParallel)
options(MulticoreParam = MulticoreParam(workers = 2))

goago <- GOaGO(genePairsGM12878,
    keyType = "ENTREZID", OrgDb = org.Hs.eg.db, ont = "ALL"
)

## ----result-------------------------------------------------------------------
head(as.data.frame(goago))

## ----dotplot, fig.cap = "Dotplot of 10 most enriched Gene Ontology terms"-----
DotPlot(goago)

## ----ridgeplot, fig.cap = "Ridgeplot of 10 most enriched Gene Ontology terms"----
RidgePlot(goago)

## ----setup-comparison, message = FALSE----------------------------------------
library(clusterProfiler)
library(ggplot2)
library(ggrepel)

## ----ego----------------------------------------------------------------------
genes <- with(
    subset(genePairsGM12878, geneID1 != geneID2),
    unique(c(geneID1, geneID2))
)

ego <- enrichGO(genes,
    keyType = "ENTREZID", OrgDb = org.Hs.eg.db, ont = "ALL"
)

## ----dotplot-ego, fig.cap = "Dotplot of 10 most enriched Gene Ontology terms, using unpaired approach by enrichGO"----
DotPlot(ego)

## ----goago-ego-relaxed--------------------------------------------------------
goago_relaxed <- GOaGO(genePairsGM12878,
    keyType = "ENTREZID", OrgDb = org.Hs.eg.db, ont = "ALL",
    pvalueCutoff = Inf, qvalueCutoff = Inf
)

ego_relaxed <- enrichGO(genes,
    keyType = "ENTREZID", OrgDb = org.Hs.eg.db, ont = "ALL",
    pvalueCutoff = Inf, qvalueCutoff = Inf
)

## ----merge-go-ego-------------------------------------------------------------
df <- merge(as.data.frame(goago_relaxed), as.data.frame(ego_relaxed),
    by = c("ONTOLOGY", "ID", "Description"), suffixes = c(".GOaGO", ".enrichGO")
)

df$signif.GOaGO <- df$ID %in% as.data.frame(goago)$ID
df$signif.enrichGO <- df$ID %in% ego$ID

df$enrichment <- ifelse(df$signif.GOaGO,
    ifelse(df$signif.enrichGO, "goago_and_ego", "goago_only"),
    ifelse(df$signif.enrichGO, "ego_only", "insignificant")
)

## ----comparison, fig.cap = "Gene Ontology term enrichment using unpaired approach by enrichGO (X-axis) and paired approach by GO-a-GO (Y-axis)", fig.wide = TRUE, fig.asp = 0.7----
col_labels <- c(
    goago_and_ego = "GO-a-GO and enrichGO",
    goago_only = "GO-a-GO (paired) only",
    ego_only = "enrichGO (unpaired) only",
    insignificant = "none of the above"
)

col_values <- c(
    goago_and_ego = "#6a3d9a",
    goago_only = "#e31a1c",
    ego_only = "#1f78b4",
    insignificant = "gray80"
)

ggplot(
    df,
    aes(x = log2(FoldEnrichment.enrichGO), y = log2(FoldEnrichment.GOaGO), size = Count.GOaGO, color = enrichment)
) +
    geom_vline(xintercept = 0, lty = 2) +
    geom_hline(yintercept = 0, lty = 2) +
    geom_point(alpha = 0.3) +
    scale_size_area() +
    scale_color_manual("Term enrichment", labels = col_labels, values = col_values) +
    DOSE::theme_dose(12)

## ----comparison-with-labels, fig.cap = "Gene Ontology term enrichment using unpaired approach by enrichGO (X-axis) and paired approach by GO-a-GO (Y-axis), showing only the terms shared by at least 5 gene pairs", fig.wide = TRUE, fig.asp = 0.7----
ggplot(
    subset(df, Count.GOaGO >= 5),
    aes(x = log2(FoldEnrichment.enrichGO), y = log2(FoldEnrichment.GOaGO), size = Count.GOaGO, color = enrichment)
) +
    geom_vline(xintercept = 0, lty = 2) +
    geom_point(alpha = 0.3) +
    geom_text_repel(aes(label = label_wrap_gen(50)(Description)), show.legend = FALSE, size = 3, lineheight = 0.9) +
    scale_size_area() +
    scale_color_manual("Term enrichment", labels = col_labels, values = col_values) +
    DOSE::theme_dose(12)

## ----termGenePairs------------------------------------------------------------
tgp <- termGenePairs(goago, org.Hs.eg.db)
subset(tgp, ID == "GO:0004984") # olfactory receptor activity

## ----num_loops_with_genes-----------------------------------------------------
length(unique(genePairsGM12878$interactionID))

## ----num_unique_gene_pairs----------------------------------------------------
nrow(genePairs(goago))

## ----show---------------------------------------------------------------------
goago

## ----import-bedpe, message = FALSE--------------------------------------------
fpath <- system.file("extdata", "GM12878_loops.bedpe.gz", package = "GOaGO")
pairs <- rtracklayer::import(fpath, genome = "hg19")
pairs

## ----transcripts, message = FALSE---------------------------------------------
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
transcripts <- transcripts(TxDb.Hsapiens.UCSC.hg19.knownGene,
    columns = "gene_id", filter = list(cds_strand = c("-", "+"))
)

## ----annotateInteractions-----------------------------------------------------
genePairs <- annotateInteractions(pairs, transcripts, maxDistanceToTSS = 10e3)
genePairs

## ----attr-keyType-------------------------------------------------------------
attr(genePairs, "keyType")

## ----citation-----------------------------------------------------------------
citation("GOaGO")

## ----sessionInfo, echo = FALSE------------------------------------------------
sessionInfo()

## Coercion to a data frame

`as.data.frame.GOaGO-result` <- function(x, ...) {
    data.frame(x@result, ...)
}

setAs("GOaGO-result", "data.frame", function(from) {
    `as.data.frame.GOaGO-result`(from)
})

##' Coerce a \code{GOaGO-result} object to a data frame
##'
##' @name as.data.frame
##' @aliases as.data.frame,GOaGO-result-method
##' @docType methods
##' @rdname as.data.frame-methods
##'
##' @usage as.data.frame(x, row.names=NULL, optional=FALSE, ...)
##' @param x The object to coerce.
##' @param row.names,optional,... Not used, inherited from
##'   \code{base::as.data.frame()}.
##' @returns A data frame of the enriched Gene Ontology terms, with the
##'   following columns: \code{ONTOLOGY}, \code{ID}, \code{Description} (all of
##'   the GO term), \code{Count} (number of input gene pairs sharing the given
##'   term), \code{PairRatio} (fraction of input gene pairs sharing the given
##'   term), \code{BgRatio} (fraction of permuted gene pairs sharing the given
##'   term), \code{FoldEnrichment} (quotient of the two fractions),
##'   \code{pvalue}, \code{p.adjust}, \code{qvalue}.
##' @examples
##' library(org.Hs.eg.db)
##' data("genePairsGM12878")
##'
##' goago <- GOaGO(genePairsGM12878, keyType = "ENTREZID", OrgDb = org.Hs.eg.db)
##' as.data.frame(goago)
setMethod(
    "as.data.frame", signature(x = "GOaGO-result"),
    `as.data.frame.GOaGO-result`
)


## Accessors

##' Accessors and show method for \code{GOaGO-result} objects
##'
##' @name GOaGO-accessors
##' @param object of class \code{GOaGO-result}
##' @returns
##' \code{genePairs} returns a data frame with the input gene pairs, with the
##' columns \code{geneID1}, \code{geneID2} and \code{pairID}.
##'
##' \code{keyType} returns the type of gene identifiers, such as "ENTREZID" or
##' "ENSEMBL".
##'
##' \code{organism} returns the scientific name (i.e. genus and species, or
##' genus and species and subspecies) of the organism.
##'
##' \code{show} displays the object, and returns an invisible \code{NULL}.
##' @examples
##' library(org.Hs.eg.db)
##' data("genePairsGM12878")
##'
##' goago <- GOaGO(genePairsGM12878, keyType = "ENTREZID", OrgDb = org.Hs.eg.db)
##' show(goago)
##'
##' genePairs(goago)
##' keyType(goago)
##' organism(goago)
NULL

##' @rdname GOaGO-accessors
##' @export
genePairs <- function(object) {
    object@genePairs
}

##' @rdname GOaGO-accessors
##' @export
keyType <- function(object) {
    object@keyType
}

##' @name organism
##' @aliases organism,GOaGO-result-method
##' @rdname GOaGO-accessors
##' @usage organism(object)
##' @export
setMethod(
    "organism", signature(object = "GOaGO-result"),
    function(object) {
        object@organism
    }
)

library(data.table)
library(GOaGO)
library(rtracklayer)
library(TxDb.Hsapiens.UCSC.hg19.knownGene)


#
#  Download from the Gene Expression Omnibus (GEO) repository a dataset
#  of 9,448 chromatin loops identified in human cell line GM12878,
#  and save it as a BEDPE file in the `inst/extdata` directory.
#

url <- paste0(
    "https://ftp.ncbi.nlm.nih.gov/geo/series/GSE63nnn/GSE63525/suppl/",
    "GSE63525_GM12878_primary%2Breplicate_HiCCUPS_looplist.txt.gz"
)

dt <- fread(url, sep = "\t", header = TRUE)
dt[, chr1 := paste0("chr", chr1)]
dt[, chr2 := paste0("chr", chr2)]

si <- GenomeInfoDb::Seqinfo(genome = "hg19")
gr1 <- makeGRangesFromDataFrame(dt,
    seqinfo = si, seqnames.field = "chr1", start.field = "x1", end.field = "x2",
    ignore.strand = TRUE, starts.in.df.are.0based = TRUE
)
gr2 <- makeGRangesFromDataFrame(dt,
    seqinfo = si, seqnames.field = "chr2", start.field = "y1", end.field = "y2",
    ignore.strand = TRUE, starts.in.df.are.0based = TRUE
)
pairs <- Pairs(gr1, gr2)

export(pairs, gzfile(file.path("..", "extdata", "GM12878_loops.bedpe.gz"), compression = 9))


#
#  Annotate the loops using gene TSSes from TxDb.Hsapiens.UCSC.hg19.knownGene,
#  and save the resulting data table with gene pairs in the `data` directory.
#

# take only the transcripts of coding genes by ensuring that the coding sequence strand is not NA
transcripts <- transcripts(TxDb.Hsapiens.UCSC.hg19.knownGene,
    columns = "gene_id", filter = list(cds_strand = c("-", "+"))
)

genePairsGM12878 <- annotateInteractions(pairs, transcripts, maxDistanceToTSS = 10e3)

save(genePairsGM12878,
    file = file.path("..", "..", "data", "genePairsGM12878.rda"),
    version = 2, compress = "xz", compression_level = 9
)

##' Extract the enriched Gene Ontology terms along with gene pairs sharing them
##'
##' @param object of class \code{GOaGO-result}
##' @param OrgDb OrgDb to map gene identifiers to gene symbols
##' @returns A data frame similar to returned by \code{as.data.frame(object)},
##'   but including all the gene pairs sharing each enriched Gene Ontology term,
##'   one gene pair in each row. Additional columns include \code{pairID},
##'   \code{geneID1}, \code{geneID2} and any other columns provided in
##'   \code{genePairs} argument to \code{\link{GOaGO}}. If \code{OrgDb} is
##'   provided, \code{geneSymbol1} and \code{geneSymbol2} will also be added.
##' @seealso \code{\link{as.data.frame,GOaGO-result-method}}
##' @export
##' @examples
##' library(org.Hs.eg.db)
##' data("genePairsGM12878")
##'
##' goago <- GOaGO(genePairsGM12878, keyType = "ENTREZID", OrgDb = org.Hs.eg.db)
##' termGenePairs(goago, OrgDb = org.Hs.eg.db)
termGenePairs <- function(object, OrgDb = NULL) {
    # first merge the enriched Gene Ontology terms with all the input gene
    # pairs that share the given term (by identifier of the GO term)
    gp <- merge(as.data.table(object@result), object@pairTerms,
        by = "ID", all = TRUE, sort = FALSE, allow.cartesian = TRUE
    )
    # then add the information on gene identifiers for each gene pair
    gp <- merge(gp, object@genePairs, by = "pairID", all.x = TRUE, sort = FALSE)

    # add gene symbols if OrgDb is provided
    if (!is.null(OrgDb)) {
        geneIDs <- unique(with(gp, c(geneID1, geneID2)))
        # suppress the message on 1:1 mapping between keys and columns
        suppressMessages(geneSymbols <- mapIds(OrgDb, geneIDs,
            column = "SYMBOL", keytype = object@keyType
        ))

        gp$geneSymbol1 <- geneSymbols[match(gp$geneID1, geneIDs)]
        gp$geneSymbol2 <- geneSymbols[match(gp$geneID2, geneIDs)]
    }

    # set the column order so that the columns from object@result will be first
    # and column order from object@genePairs will also be preserved
    setcolorder(gp, unique(c(names(object@result), names(object@genePairs))))

    return(gp)
}

##' An S4 class to represent the results of GO-a-GO enrichment analysis
##'
##' @slot result A data frame of the enriched Gene Ontology terms, with the
##'   following columns: \code{ONTOLOGY}, \code{ID}, \code{Description} (all of
##'   the GO term), \code{Count} (number of input gene pairs sharing the given
##'   term), \code{PairRatio} (fraction of input gene pairs sharing the given
##'   term), \code{BgRatio} (fraction of permuted gene pairs sharing the given
##'   term), \code{FoldEnrichment} (quotient of the two fractions),
##'   \code{pvalue}, \code{p.adjust}, \code{qvalue}.
##' @slot pvalueCutoff adjusted p-value cutoff on enrichment tests
##' @slot pAdjustMethod p-value adjustment method
##' @slot qvalueCutoff q-value cutoff on enrichment tests
##' @slot minCount cutoff for number of pairs that share a GO term for this term
##'   to be considered
##' @slot numPermutations number of permutations performed in the enrichment
##'   test
##' @slot minGSSize minimal size of genes annotated for testing
##' @slot maxGSSize maximal size of genes annotated for testing
##' @slot organism scientific name of the organism
##' @slot ontology one of "BP", "MF", and "CC" subontologies, or "ALL" for all
##'   three
##' @slot keyType type of gene identifiers, such as "ENTREZID" or "ENSEMBL"
##' @slot genePairs A data frame with the input gene pairs, with the columns
##'   \code{geneID1}, \code{geneID2} and \code{pairID}.
##' @slot pairTerms A data frame linking the enriched Gene Ontology terms with
##'   the input gene pairs, with the columns \code{pairID} and \code{ID} (of the
##'   GO term).
##' @slot permutedResult A data frame with the columns \code{ID} (of the GO
##'   term) and \code{Count}, keeping the numbers of permuted gene pairs sharing
##'   the term as obtained in every random permutation.
##' @slot universe a set of background genes
##' @export
setClass("GOaGO-result",
    slots = c(
        result          = "data.frame",
        pvalueCutoff    = "numeric",
        pAdjustMethod   = "character",
        qvalueCutoff    = "numeric",
        minCount        = "numeric",
        numPermutations = "numeric",
        minGSSize       = "numeric",
        maxGSSize       = "numeric",
        organism        = "character",
        ontology        = "character",
        keyType         = "character",
        genePairs       = "data.frame",
        pairTerms       = "data.frame",
        permutedResult  = "data.frame",
        universe        = "character"
    )
)


##' Extract unique gene pairs from the data frame provided
##'
##' Given a data frame of gene pairs, this function will return the unique
##' pairs of genes, removing loops (gene pairs containing the same gene twice)
##' and duplicates. Note that gene pair (A, B) is a duplicate of (B, A).
##'
##' @param genePairs a data frame with columns \code{geneID1} and \code{geneID2}
##'   containing gene identifiers; column \code{pairID} will also be used if
##'   provided.
##' @returns A data frame with columns \code{pairID}, \code{geneID1} and
##'   \code{geneID2}. If loops or duplicates were removed, a warning will
##'   alert you. If column \code{pairID} was not provided in \code{genePairs},
##'   an integer vector equal to \code{seq_len(nrow(result))} will be used.
uniqueGenePairs <- function(genePairs) {
    # prevent "no visible binding for global variable" NOTEs in R CMD check
    geneID1 <- geneID2 <- gid1 <- gid2 <- pairID <- NULL

    # ensure that gene identifiers are provided in the input data
    stopifnot("geneID1" %in% colnames(genePairs))
    stopifnot("geneID2" %in% colnames(genePairs))

    # ensure that pair identifiers are unique if provided
    pairID_provided <- "pairID" %in% colnames(genePairs)
    if (pairID_provided) {
        stopifnot(!anyDuplicated(genePairs$pairID))
    }
    genePairs <- as.data.table(genePairs)

    n <- nrow(genePairs)
    genePairs[, gid1 := pmin(geneID1, geneID2)]
    genePairs[, gid2 := pmax(geneID1, geneID2)]
    genePairs <- unique(genePairs, by = c("gid1", "gid2"))
    if (nrow(genePairs) != n) {
        warning("removing ", n - nrow(genePairs), " duplicated gene pair(s)")
    }
    genePairs[, gid1 := NULL]
    genePairs[, gid2 := NULL]

    n <- nrow(genePairs)
    genePairs <- genePairs[geneID1 != geneID2, ]
    if (nrow(genePairs) != n) {
        warning(
            "removing ", n - nrow(genePairs),
            " gene pair(s) containing the same gene twice"
        )
    }

    if (!pairID_provided) {
        genePairs[, pairID := .I]
        setcolorder(genePairs, "pairID")
    }

    return(genePairs)
}


##' Gene Ontology enrichment analysis in a set of gene pairs
##'
##' Given a data frame of gene pairs, this function will return the enriched
##' Gene Ontology terms after FDR control.
##'
##' @param genePairs a data frame with columns \code{geneID1} and \code{geneID2}
##'   containing gene identifiers; column \code{pairID} will also be used if
##'   provided.
##' @param OrgDb OrgDb
##' @param keyType type of gene identifiers, such as "ENTREZID" or "ENSEMBL", if
##'   it cannot be determined from metadata of \code{genePairs}
##' @param ont one of "BP", "MF", and "CC" subontologies, or "ALL" for all three
##' @param minCount cutoff for number of pairs that share a GO term for this
##'   term to be considered
##' @param numPermutations number of permutations performed in the enrichment
##'   test
##' @param universe a set of background genes. If missing, all the genes from
##'   all the gene pairs will be used as background.
##' @param pvalueCutoff adjusted p-value cutoff on enrichment tests to report
##' @param pAdjustMethod one of "holm", "hochberg", "hommel", "bonferroni",
##'   "BH", "BY", "fdr", "none"
##' @param qvalueCutoff q-value cutoff on enrichment tests to report as
##'   significant. Tests must pass (i) \code{pvalueCutoff} on unadjusted
##'   p-values, (ii) \code{pvalueCutoff} on adjusted p-values, and (iii)
##'   \code{qvalueCutoff} on q-values to be reported.
##' @param minGSSize minimal size of genes annotated for testing
##' @param maxGSSize maximal size of genes annotated for testing
##' @returns A \code{GOaGO-result} object.
##' @seealso \code{\link{GOaGO-result-class}}
##' @export
##' @examples
##' library(org.Hs.eg.db)
##' data("genePairsGM12878")
##'
##' goago <- GOaGO(genePairsGM12878, keyType = "ENTREZID", OrgDb = org.Hs.eg.db)
##' show(goago)
GOaGO <- function(
    genePairs, OrgDb, keyType = NULL, ont = "MF",
    minCount = 1, numPermutations = 10000, universe, pvalueCutoff = 0.05,
    pAdjustMethod = "BH", qvalueCutoff = 0.2, minGSSize = 10, maxGSSize = 500
) {
    # prevent "no visible binding for global variable" NOTEs in R CMD check
    geneID <- ID <- FoldEnrichment <- PairRatio <- BgRatio <- pvalue <- NULL

    # extract unique gene pairs from the data frame provided
    genePairs <- uniqueGenePairs(genePairs)
    # ensure that gene identifiers are character vectors
    genePairs$geneID1 <- as.character(genePairs$geneID1)
    genePairs$geneID2 <- as.character(genePairs$geneID2)

    # gene universe
    if (missing(universe)) {
        # if not explicitly provided, universe is constituted by all the genes
        # from all the gene pairs
        universe <- c(genePairs$geneID1, genePairs$geneID2)
    } else {
        # ensure that gene identifiers are character vectors
        universe <- as.character(universe)
        # check if all the genes are within the universe
        stopifnot(genePairs$geneID1 %in% universe)
        stopifnot(genePairs$geneID2 %in% universe)
    }
    # remove duplicates from the universe
    universe <- unique(universe)

    # use keyType if provided, otherwise try to determine it from genePairs
    if (is.null(keyType)) {
        keyType <- attr(genePairs, "keyType")
        if (is.null(keyType)) {
            stop(
                "type of gene identifiers cannot be determined from metadata ",
                "of `genePairs`, please provide it as `keyType` argument"
            )
        }
    }

    # use enrichGO to fetch the information on gene-to-term associations
    ego <- enrichGO(
        gene = universe, OrgDb = OrgDb, keyType = keyType,
        ont = ont, pAdjustMethod = "none", pvalueCutoff = Inf,
        qvalueCutoff = Inf, minGSSize = minGSSize, maxGSSize = maxGSSize
    )
    egoResult <- as.data.table(ego)

    # GO term universe
    ID_universe <- egoResult$ID
    stopifnot(!anyDuplicated(ID_universe))
    # take only the relevant columns
    go_split <- egoResult[,
        list(geneID = factor(strsplit(geneID, "/")[[1]], universe)),
        by = list(ID = factor(ID, ID_universe)),
        showProgress = FALSE
    ]
    stopifnot(as.integer(go_split$geneID) == match(go_split$geneID, universe))
    stopifnot(!is.na(go_split$ID))

    # matrix (not a data frame) defining the gene pairs
    genePairsMatrix <- cbind(
        factor(genePairs$geneID1, universe),
        factor(genePairs$geneID2, universe)
    )

    # construct sparse gene x term matrix
    geneTermMatrix <- Matrix(FALSE,
        nrow = length(universe),
        ncol = length(ID_universe), sparse = TRUE, doDiag = FALSE
    )
    ind <- cbind(as.integer(go_split$geneID), as.integer(go_split$ID))
    geneTermMatrix[ind] <- TRUE

    # a function useful for permutation testing
    countGenePairsPerTerm <- function(genePairsMatrix,
        geneTermMatrix,
        permuted = FALSE
    ) {
        if (permuted) {
            permuted_genes <- sample.int(length(universe))
            gene1 <- permuted_genes[genePairsMatrix[, 1]]
            gene2 <- permuted_genes[genePairsMatrix[, 2]]
        } else {
            gene1 <- genePairsMatrix[, 1]
            gene2 <- genePairsMatrix[, 2]
        }

        bw <- geneTermMatrix[gene1, , drop = FALSE] &
            geneTermMatrix[gene2, , drop = FALSE]
        result <- as.integer(colSums(bw))
        return(result)
    }

    # simplified version of the above, without permutations and per-term
    # aggregation
    pairTermsMatrix <- function(genePairsMatrix, geneTermMatrix) {
        gene1 <- genePairsMatrix[, 1]
        gene2 <- genePairsMatrix[, 2]
        bw <- geneTermMatrix[gene1, , drop = FALSE] &
            geneTermMatrix[gene2, , drop = FALSE]
        return(bw)
    }

    pairCountsPerTerm <- countGenePairsPerTerm(genePairsMatrix, geneTermMatrix)

    # take only the GO terms that are associated to at least minCount gene pairs
    sel <- which(pairCountsPerTerm >= minCount)
    ID_universe_reduced <- ID_universe[sel]
    pairCountsPerTerm_reduced <- pairCountsPerTerm[sel]
    geneTermMatrix_reduced <- geneTermMatrix[, sel, drop = FALSE]

    # now do the permutation test: for each permutation, count permuted gene
    # pairs sharing each GO term
    countPermutedGenePairsPerTerm <- function(i) {
        result <- countGenePairsPerTerm(genePairsMatrix, geneTermMatrix_reduced,
            permuted = TRUE
        )
        return(result)
    }
    # store the resulting counts in term x permutation matrix
    permutedPairCountsPerTerm_reduced <- do.call(
        cbind,
        bplapply(seq_len(numPermutations), countPermutedGenePairsPerTerm)
    )

    # aggregate the results of permutation testing
    cols <- intersect(c("ONTOLOGY", "ID", "Description"), colnames(egoResult))
    result <- cbind(
        egoResult[sel, cols, with = FALSE],
        Count = pairCountsPerTerm_reduced,
        PairRatio = pairCountsPerTerm_reduced / nrow(genePairsMatrix),
        BgRatio = rowMeans(permutedPairCountsPerTerm_reduced) /
            nrow(genePairsMatrix)
    )
    result[, FoldEnrichment := PairRatio / BgRatio]
    result$pvalue <- rowMeans(permutedPairCountsPerTerm_reduced >=
        pairCountsPerTerm_reduced)

    # adjust p-values and estimate q-values
    result[, p.adjust := p.adjust(pvalue, method = pAdjustMethod)]
    qobj <- tryCatch(qvalue(
        p = result$pvalue, lambda = 0.05,
        pi0.method = "bootstrap"
    ), error = function(e) NULL)
    if (inherits(qobj, "qvalue")) {
        result[, qvalue := qobj$qvalues]
        sel_signif <- which(result$pvalue < pvalueCutoff &
            result$p.adjust <= pvalueCutoff & result$qvalue <= qvalueCutoff)
    } else {
        result[, qvalue := NA]
        sel_signif <- which(result$pvalue < pvalueCutoff &
            result$p.adjust <= pvalueCutoff)
    }

    # take only the GO terms that are significantly overrepresented
    result <- result[sel_signif, ]
    ID_universe_reduced_signif <- ID_universe_reduced[sel_signif]
    geneTermMatrix_reduced_signif <- geneTermMatrix_reduced[, sel_signif,
        drop = FALSE
    ]
    permutedPairCountsPerTerm_reduced_signif <-
        permutedPairCountsPerTerm_reduced[sel_signif, , drop = FALSE]

    # construct sparse pair x term matrix, convert it to tidy format
    pt <- which(pairTermsMatrix(genePairsMatrix, geneTermMatrix_reduced_signif),
        arr.ind = TRUE
    )
    pairTerms <- data.table(
        pairID = genePairs$pairID[pt[, 1]],
        ID = factor(ID_universe_reduced_signif)[pt[, 2]]
    )

    # convert the results of the permutations to tidy format
    ind <- as.vector(row(permutedPairCountsPerTerm_reduced_signif))
    permutedResult <- data.table(
        ID = factor(ID_universe_reduced_signif)[ind],
        Count = as.vector(permutedPairCountsPerTerm_reduced_signif)
    )

    # sort the results according to p-value
    result <- result[order(pvalue), ]

    result <- new("GOaGO-result",
        result = result,
        pvalueCutoff = pvalueCutoff,
        pAdjustMethod = pAdjustMethod,
        qvalueCutoff = qvalueCutoff,
        minCount = minCount,
        numPermutations = numPermutations,
        minGSSize = minGSSize,
        maxGSSize = maxGSSize,
        organism = species(OrgDb),
        ontology = ont,
        keyType = keyType,
        genePairs = genePairs,
        pairTerms = pairTerms,
        permutedResult = permutedResult,
        universe = universe
    )
    return(result)
}

test_that("`uniqueGenePairs` should throw a warning when loops (gene pairs
containing the same gene twice) are found in the `genePairs` argument", {
    genePairs <- data.table(geneID1 = c("g1", "g3"), geneID2 = c("g2", "g3"))
    expect_warning(
        genePairs <- uniqueGenePairs(genePairs),
        "removing 1 gene pair\\(s\\) containing the same gene twice"
    )
    expect_equal(
        genePairs,
        data.table(pairID = 1, geneID1 = "g1", geneID2 = "g2")
    )
})

test_that("`uniqueGenePairs` should throw a warning when duplicate gene pairs
are found in the `genePairs` argument", {
    genePairs <- data.table(geneID1 = c("g1", "g2"), geneID2 = c("g2", "g1"))
    expect_warning(
        genePairs <- uniqueGenePairs(genePairs),
        "removing 1 duplicated gene pair\\(s\\)"
    )
    expect_equal(
        genePairs,
        data.table(pairID = 1, geneID1 = "g1", geneID2 = "g2")
    )
})


#' Construct a simulated set of gene pairs for testing GOaGO.
#'
#' The result will consist of \code{num_term1_pairs} pairs between genes that
#' have \code{term1}, but do not have \code{term2}, and \code{num_term2_pairs}
#' pairs between genes that have \code{term2}, but do not have \code{term1}. The
#' genes will be sampled from a given genome-wide annotation database.
#'
#' @param OrgDb OrgDb from which the genes will be sampled
#' @param term1 GO term 1
#' @param term2 GO term 2
#' @param num_term1_pairs number of pairs between genes having \code{term1}
#' @param num_term2_pairs number of pairs between genes having \code{term2}
#' @param keyType type of gene identifiers to use, such as "ENTREZID" or "ENSEMBL"
#'
#' @returns A data frame with columns \code{geneID1} and \code{geneID2}.
simulate_gene_pairs <- function(
    OrgDb, term1, term2, num_term1_pairs, num_term2_pairs, keyType = "ENTREZID"
) {
    results1 <- AnnotationDbi::select(OrgDb, keys = term1, columns = c(keyType), keytype = "GOALL")
    results2 <- AnnotationDbi::select(OrgDb, keys = term2, columns = c(keyType), keytype = "GOALL")

    candidate_genes1 <- unique(results1[[keyType]])
    candidate_genes2 <- unique(results2[[keyType]])

    genes1 <- setdiff(candidate_genes1, candidate_genes2)
    genes2 <- setdiff(candidate_genes2, candidate_genes1)

    stopifnot(num_term1_pairs > 0)
    stopifnot(num_term2_pairs > 0)
    stopifnot(length(genes1) >= 2 * num_term1_pairs)
    stopifnot(length(genes2) >= 2 * num_term2_pairs)

    genePairs <- data.frame(
        geneID1 = c(
            genes1[seq.int(from = 1, by = 2, length.out = num_term1_pairs)],
            genes2[seq.int(from = 1, by = 2, length.out = num_term2_pairs)]
        ),
        geneID2 = c(
            genes1[seq.int(from = 2, by = 2, length.out = num_term1_pairs)],
            genes2[seq.int(from = 2, by = 2, length.out = num_term2_pairs)]
        )
    )

    return(genePairs)
}

##' Parameters to construct simulated gene pairs for testing
term1 <- "GO:0003677" # DNA binding
term2 <- "GO:0046983" # protein dimerization activity
num_term1_pairs <- 10
num_term2_pairs <- 20
keyType <- "ENTREZID"

##' Note that it would be challenging to test the p-values and enrichment ratios
##' obtained from random permutations, so we do not check the values in columns
##' `BgRatio` and `pvalue`. To ensure that the results are not filtered out by
##' p-value or q-value, we set `pvalueCutoff` and `qvalueCutoff` to permissive
##' values.
##'
##' The GO terms chosen for this test are unlikely to ever become obsolete, but
##' the number of genes having this term (Gene Set Size) will surely change.
##' Hence, we set `minGSSize` and `maxGSSize` so that no term is excluded.

test_that("`GOaGO` should return a `GOaGO-result` instance, with statistics
(Count, PairRatio) consistent with the simulated data", {
    testthat::skip_if_not_installed("org.Hs.eg.db")
    genePairs <- simulate_gene_pairs(
        org.Hs.eg.db::org.Hs.eg.db, term1, term2,
        num_term1_pairs, num_term2_pairs, keyType
    )
    goago <- GOaGO(genePairs,
        keyType = keyType, OrgDb = org.Hs.eg.db::org.Hs.eg.db,
        pvalueCutoff = Inf, qvalueCutoff = Inf, minGSSize = 0, maxGSSize = Inf
    )
    expect_s4_class(goago, "GOaGO-result")
    expect_equal(subset(goago@result, ID == term1)$Count, num_term1_pairs)
    expect_equal(subset(goago@result, ID == term2)$Count, num_term2_pairs)
    expect_equal(
        subset(goago@result, ID == term1)$PairRatio,
        num_term1_pairs / (num_term1_pairs + num_term2_pairs)
    )
    expect_equal(
        subset(goago@result, ID == term2)$PairRatio,
        num_term2_pairs / (num_term1_pairs + num_term2_pairs)
    )
})

# find the metadata column containing gene identifiers, following the convention
# of GenomicRanges::makeGRangesFromDataFrame()
.find_geneid_col <- function(metadata_colnames, geneid_column) {
    ind <- which(metadata_colnames %in% geneid_column)
    if (length(ind) == 0) {
        stop("cannot find gene ID column")
    }
    if (length(ind) >= 2) {
        stop("cannot determine gene ID column unambiguously")
    }
    return(ind)
}


##' Convert gene transcripts to Transcription Start Sites
##'
##' Convert a \code{GRanges} object with gene transcripts to a \code{GRanges}
##' object with gene TSSes of length 1 bp, with duplicate rows removed. Assumes
##' that gene identifiers are provided in one of the metadata columns, either as
##' a vector (possibly containing \code{NA} values) or a \code{CharacterList}.
##' The function is idempotent, i.e. can be applied multiple times without
##' changing the result.
##'
##' @param transcripts \code{TxDb} annotation or other \code{GRanges} object
##' @param geneid_column A character vector of recognized names for the metadata
##' column in \code{transcripts} that contains gene identifiers. If none or more
##' than one is found, an error is raised.
##'
##' @returns A \code{GRanges} object with gene identifiers in metadata column
##' \code{geneID} being a vector.
##' @export
##'
##' @examples
##' library(TxDb.Hsapiens.UCSC.hg19.knownGene)
##'
##' # take only the transcripts of coding genes by ensuring that the coding
##' # sequence strand is not NA
##' transcripts <- transcripts(TxDb.Hsapiens.UCSC.hg19.knownGene,
##'     columns = "gene_id", filter = list(cds_strand = c("-", "+"))
##' )
##'
##' convertTranscriptsToTSS(transcripts)
convertTranscriptsToTSS <- function(
    transcripts, geneid_column = c("gene_id", "GENEID", "geneID")
) {
    # find the metadata column with gene IDs
    geneid_col <- .find_geneid_col(names(mcols(transcripts)), geneid_column)
    geneid_values <- mcols(transcripts)[[geneid_col]]

    # convert transcripts to TSSes
    tss <- resize(transcripts, width = 1, fix = "start")
    # remove metadata columns from the resulting GRanges object
    mcols(tss) <- NULL

    # repopulate gene ID column in the metadata, unlisting a CharacterList
    # or removing NAs from an atomic vector as needed
    if (inherits(geneid_values, "List")) {
        ind <- rep(seq_along(tss), vapply(geneid_values, length, integer(1)))
        tss <- tss[ind]
        tss$geneID <- unlist(geneid_values)
    } else {
        stopifnot(is.vector(geneid_values))
        ind <- !is.na(geneid_values)
        tss <- tss[ind]
        tss$geneID <- geneid_values[ind]
    }

    # remove duplicate rows; here the conversion to a data table is to avoid
    # collapsing TSSes of different genes (geneID column is in the metadata)
    tss <- tss[!duplicated(as.data.table(tss)), ]

    return(tss)
}


# use pre-computed TSSes if provided, otherwise use convertTranscriptsToTSS
.determine_tss <- function(transcripts = NULL, tss = NULL) {
    if (is.null(tss)) {
        if (is.null(transcripts)) {
            stop("either `transcripts` or `tss` must be provided")
        }
        tss <- convertTranscriptsToTSS(transcripts)
    } else {
        if (!is.null(transcripts)) {
            stop("`transcripts` and `tss` cannot be both provided")
        }
        stopifnot(width(tss) == 1)
        stopifnot(is.vector(tss$geneID))
    }

    return(tss)
}


# determine keyType from GRanges metadata
.determine_keyType <- function(tss, keyType = NULL) {
    # if a value is provided, use it
    if (!is.null(keyType)) {
        return(keyType)
    }

    keyType_description <- metadata(tss)$genomeInfo$`Type of Gene ID`
    # the above can be NULL if there is no genomeInfo in the metadata etc.
    if (is.null(keyType_description)) {
        keyType_description <- ""
    }

    if (keyType_description == "Entrez Gene ID") {
        return("ENTREZID")
    } else if (keyType_description == "Ensembl gene ID") {
        return("ENSEMBL")
    } else {
        stop(
            "type of gene identifiers cannot be determined from the metadata, ",
            "please provide it as `keyType` argument"
        )
    }
}


##' Associate interaction anchors to the nearest TSSes
##'
##' Interaction anchors are associated to TSSes as follows. Each anchor is
##' associated to all the TSSes it overlaps. If there is no such overlap, then
##' the anchor is associated to all TSSes with the shortest distance to the
##' anchor, if this distance is not larger than \code{maxDistanceToTSS}.
##'
##' Either \code{transcripts} or \code{tss} must be provided, but not both.
##'
##' @param anchors object of class \code{GRanges}
##' @param transcripts \code{TxDb} annotation or other \code{GRanges} object
##' @param tss object of class \code{GRanges} as returned by
##'   \code{\link{convertTranscriptsToTSS}}
##' @param keyType type of gene identifiers, such as "ENTREZID" or "ENSEMBL", if
##'   it cannot be determined from metadata of \code{transcripts} or \code{tss}
##' @param maxDistanceToTSS maximal distance to extend the search for nearest
##'   TSS outside the anchor
##'
##' @returns A data table with columns \code{interactionID} (index of the
##'   anchor), \code{chrom}, \code{start}, \code{end} (coordinates of the
##'   anchor), \code{geneID} (gene identifier from `transcripts` or `tss`),
##'   \code{tss} (TSS position) and \code{strand} (TSS strand).
##' @seealso \code{\link{convertTranscriptsToTSS}}
##' @export
##'
##' @examples
##' library(TxDb.Hsapiens.UCSC.hg19.knownGene)
##'
##' # take only the transcripts of coding genes by ensuring that the coding
##' # sequence strand is not NA
##' transcripts <- transcripts(TxDb.Hsapiens.UCSC.hg19.knownGene,
##'     columns = "gene_id", filter = list(cds_strand = c("-", "+"))
##' )
##'
##' tss <- convertTranscriptsToTSS(transcripts)
##' gr <- GRanges("chr1", IRanges(c(42001, 890001), c(62000, 900000)))
##'
##' # note that anchors are associated to TSSes outside the anchor only if there
##' # are no TSSes overlapping the anchor
##' annotateAnchors(gr, tss, maxDistanceToTSS = 10e3)
##'
##' # this may yield more associations to TSSes outside the anchors
##' annotateAnchors(gr + 10e3, tss)
annotateAnchors <- function(
    anchors, transcripts = NULL, tss = NULL, keyType = NULL,
    maxDistanceToTSS = 0
) {
    # prevent "no visible binding for global variable" NOTEs in R CMD check
    distance_to_tss <- NULL

    # find all nearest TSSes for each anchor
    tss <- .determine_tss(transcripts, tss)
    ov <- nearest(anchors, tss, select = "all")

    # keep information on anchor coordinates, distance to the nearest TSS,
    # as well as gene ID, position (1-based) and strand of the nearest TSS
    nearest_tss <- data.table(
        interactionID = queryHits(ov),
        chrom = as.vector(seqnames(anchors))[queryHits(ov)],
        start = start(anchors)[queryHits(ov)],
        end = end(anchors)[queryHits(ov)],
        distance_to_tss = distance(anchors[queryHits(ov)],
            tss[subjectHits(ov)]),
        geneID = tss$geneID[subjectHits(ov)],
        tss = start(tss)[subjectHits(ov)],
        strand = as.vector(strand(tss))[subjectHits(ov)]
    )

    # keep only the TSSes not further away than maxDistanceToTSS
    nearest_tss <- nearest_tss[distance_to_tss <= maxDistanceToTSS, ]
    nearest_tss$distance_to_tss <- NULL

    # keep seqinfo and keyType from gene TSSes as attributes
    attr(nearest_tss, "seqinfo") <- seqinfo(tss)
    attr(nearest_tss, "keyType") <- .determine_keyType(tss, keyType)

    return(nearest_tss)
}


##' Associate interactions to gene pairs
##'
##' Both interaction anchors are associated to TSSes as described in
##' \code{\link{annotateAnchors}}. Briefly, each anchor is associated to all the
##' TSSes it overlaps, or to all closest TSSes up to \code{maxDistanceToTSS} if
##' there is no such overlap. The annotation of an interaction is a Cartesian
##' product of annotations for both anchors.
##'
##' Either \code{transcripts} or \code{tss} must be provided, but not both.
##'
##' @param interactions object of class \code{Pairs} (of \code{GRanges}) or
##'   \code{GenomicInteractions}
##' @param transcripts \code{TxDb} annotation or other \code{GRanges} object
##' @param tss object of class \code{GRanges} as returned by
##'   \code{\link{convertTranscriptsToTSS}}
##' @param keyType type of gene identifiers, such as "ENTREZID" or "ENSEMBL", if
##'   it cannot be determined from metadata of \code{transcripts} or \code{tss}
##' @param maxDistanceToTSS maximal distance to extend the search for nearest
##'   TSS outside the anchor
##'
##' @returns A data table with columns \code{interactionID} (index of the
##'   interaction), \code{chrom1}, \code{start1}, \code{end1}, \code{chrom2},
##'   \code{start2}, \code{end2} (coordinates of both anchors), \code{geneID1},
##'   \code{geneID2} (gene identifiers from `transcripts` or `tss` for both
##'   anchors), \code{tss1}, \code{tss2} (TSS position for both anchors),
##'   \code{strand1} and \code{strand2} (TSS strand for both anchors).
##' @seealso \code{\link{convertTranscriptsToTSS}},
##'   \code{\link{annotateAnchors}}
##' @export
##'
##' @examples
##' library(TxDb.Hsapiens.UCSC.hg19.knownGene)
##'
##' # take only the transcripts of coding genes by ensuring that the coding
##' # sequence strand is not NA
##' transcripts <- transcripts(TxDb.Hsapiens.UCSC.hg19.knownGene,
##'     columns = "gene_id", filter = list(cds_strand = c("-", "+"))
##' )
##'
##' fpath <- system.file("extdata", "GM12878_loops.bedpe.gz", package = "GOaGO")
##' pairs <- rtracklayer::import(fpath, genome = "hg19")
##' annotateInteractions(pairs, transcripts, maxDistanceToTSS = 10e3)
annotateInteractions <- function(
    interactions, transcripts = NULL, tss = NULL, keyType = NULL,
    maxDistanceToTSS = 0
) {
    # extract the two interacting regions
    if (inherits(interactions, "Pairs")) {
        # core Bioconductor class, used by package `rtracklayer`
        gr1 <- S4Vectors::first(interactions)
        gr2 <- S4Vectors::second(interactions)
    } else if (inherits(interactions, "GenomicInteractions")) {
        # used by packages `GenomicInteractions` and `mariner`
        if (!requireNamespace("GenomicInteractions", quietly = TRUE)) {
            stop("Package \"GenomicInteractions\" must be installed ",
                "to handle GenomicInteractions objects.",
                call. = FALSE
            )
        }
        gr1 <- GenomicInteractions::anchorOne(interactions)
        gr2 <- GenomicInteractions::anchorTwo(interactions)
    } else {
        stop("unknown class of `interactions` object")
    }

    # annotate the anchors by the overlapping or nearest TSSes
    tss <- .determine_tss(transcripts, tss)
    anchor1 <- annotateAnchors(gr1,
        tss = tss, keyType = keyType,
        maxDistanceToTSS = maxDistanceToTSS
    )
    anchor2 <- annotateAnchors(gr2,
        tss = tss, keyType = keyType,
        maxDistanceToTSS = maxDistanceToTSS
    )

    # construct a Cartesian product of annotations for both anchors
    genePairs <- merge(anchor1, anchor2,
        by = "interactionID",
        allow.cartesian = TRUE, suffixes = c("1", "2")
    )
    setkey(genePairs, NULL)

    # keep seqinfo and keyType from gene TSSes as attributes
    attr(genePairs, "seqinfo") <- seqinfo(tss)
    attr(genePairs, "keyType") <- .determine_keyType(tss, keyType)

    return(genePairs)
}

library(TxDb.Hsapiens.UCSC.hg19.knownGene)

# take only the transcripts of coding genes by ensuring that the coding
# sequence strand is not NA
transcripts <- transcripts(TxDb.Hsapiens.UCSC.hg19.knownGene,
    columns = "gene_id", filter = list(cds_strand = c("-", "+"))
)

fpath <- system.file("extdata", "GM12878_loops.bedpe.gz", package = "GOaGO")
pairs <- rtracklayer::import(fpath, genome = "hg19")


test_that("`convertTranscriptsToTSS` should return a GRanges object of width 1
with gene identifiers in metadata column `geneID` being a character vector", {
    tss <- convertTranscriptsToTSS(transcripts)
    expect_s4_class(tss, "GRanges")
    expect_setequal(width(tss), 1)
    expect_type(tss$geneID, "character")
})

test_that("`convertTranscriptsToTSS` should be idempotent", {
    tss <- convertTranscriptsToTSS(transcripts)
    expect_equal(tss, convertTranscriptsToTSS(tss))
})


# synthetic transcripts
st <- GRanges("chr1",
    IRanges(c(11, 11, 21, 51), c(20, 40, 40, 70)),
    strand = c("+", "+", "+", "-"),
    gene_id = CharacterList("A", c("A", "B"), c("A", "B"), "C")
)

st2 <- GRanges("chr1",
    IRanges(c(11, 11, 21, 51), c(20, 40, 40, 70)),
    strand = c("+", "+", "+", "-"),
    gene_id = c("A", "A", NA, "C")
)

test_that("`convertTranscriptsToTSS` works as expected on synthetic data", {
    expect_equal(
        convertTranscriptsToTSS(st),
        GRanges("chr1",
            IRanges(c(11, 11, 21, 21, 70)),
            strand = c("+", "+", "+", "+", "-"),
            geneID = c("A", "B", "A", "B", "C")
        )
    )
    expect_equal(
        convertTranscriptsToTSS(st2),
        GRanges("chr1",
            IRanges(c(11, 70)),
            strand = c("+", "-"),
            geneID = c("A", "C")
        )
    )
})


# synthetic anchors: the first one does not overlap a TSS, but is equidistant
# to three; the second one overlaps a TSS
sa <- GRanges("chr1", IRanges(c(31, 61), c(60, 90)))

test_that("`annotateAnchors` works as expected on synthetic data", {
    dt <- data.table(
        interactionID = 2, chrom = "chr1", start = 61, end = 90,
        geneID = "C", tss = 70, strand = "-"
    )
    attr(dt, "seqinfo") <- seqinfo(st)
    attr(dt, "keyType") <- "ENTREZID"
    expect_equal(annotateAnchors(sa, st, keyType = "ENTREZID"), dt)

    dt2 <- data.table(
        interactionID = c(1, 1, 1, 2), chrom = "chr1",
        start = c(31, 31, 31, 61), end = c(60, 60, 60, 90),
        geneID = c("A", "B", "C", "C"), tss = c(21, 21, 70, 70),
        strand = c("+", "+", "-", "-")
    )
    attr(dt2, "seqinfo") <- seqinfo(st)
    attr(dt2, "keyType") <- "ENTREZID"
    expect_equal(
        annotateAnchors(sa, st, keyType = "ENTREZID", maxDistanceToTSS = 100),
        dt2
    )
})

test_that("`annotateInteractions` works as expected on synthetic data", {
    dt <- data.table(
        interactionID = c(1, 1, 1), chrom1 = "chr1", start1 = c(31, 31, 31),
        end1 = c(60, 60, 60), geneID1 = c("A", "B", "C"), tss1 = c(21, 21, 70),
        strand1 = c("+", "+", "-"), chrom2 = "chr1", start2 = 61, end2 = 90,
        geneID2 = "C", tss2 = 70, strand2 = "-"
    )
    attr(dt, "seqinfo") <- seqinfo(st)
    attr(dt, "keyType") <- "ENTREZID"

    expect_equal(
        annotateInteractions(Pairs(sa[1], sa[2]), st,
            keyType = "ENTREZID",
            maxDistanceToTSS = 100
        ),
        dt
    )
})


test_that("`annotateInteractions` should return the same result for `Pairs` and
`GenomicInteractions` objects", {
    result1 <- annotateInteractions(pairs, transcripts, maxDistanceToTSS = 10e3)

    gi <- GenomicInteractions::GenomicInteractions(
        S4Vectors::first(pairs),
        S4Vectors::second(pairs)
    )
    result2 <- annotateInteractions(gi, transcripts, maxDistanceToTSS = 10e3)
    expect_equal(result1, result2)
})

---
title: "Gene Ontology enrichment analysis of gene pairs with GO-a-GO"
author: 
  - name: Aleksander Jankowski
output: 
  BiocStyle::html_document:
    self_contained: yes
    toc: true
    toc_depth: 2
date: "`r doc_date()`"
package: "`r pkg_ver('GOaGO')`"
vignette: >
  %\VignetteIndexEntry{Gene Ontology enrichment analysis of gene pairs with GO-a-GO}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}  
bibliography: references.bib
biblio-style: apalike
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "##"
)
```


# Introduction

`r Biocpkg("GOaGO")`, also spelled GO-a-GO, is an `R` package for Gene Ontology enrichment analysis in a given set of gene pairs.
The identification of overrepresented Gene Ontology (GO) terms in a set of genes is a standard approach to obtain functional associations, e.g. to characterize the set of differentially expressed genes between treatment and control samples.
The approach of GO-a-GO allows to identify overrepresented GO terms that are shared by both genes in a pair, given a set of gene pairs.
This provides the opportunity to annotate which functions are associated with gene pairs defined by a selected group of chromatin contacts, such as differential contacts between cell types or chromatin loops.

GO-a-GO calculates enrichment from a permutation test for overrepresentation of gene pairs that are associated with a shared GO term.
Such gene pairs are counted for the original set of gene pairs and compared against randomized sets in which the structure of the pairs is preserved, but the gene identities (including the associated GO terms) are permuted.
Therefore, we do not focus on the fact that the term is enriched in a group of genes, but that genes with this term get paired more often than expected.


# Installation

`r Biocpkg("GOaGO")` is an `R` package distributed as part of the Bioconductor [Bioconductor](http://bioconductor.org) project.
To install GO-a-GO along with any package dependencies that are missing, use `r Biocpkg("BiocManager")`:

```{r install, eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
}

BiocManager::install("GOaGO")
```


# Quick start to using GO-a-GO

First, load the package:

```{r start, message = FALSE}
library(GOaGO)
```

## Example dataset of gene pairs

We will use an example dataset based on 9,448 chromatin loops [@rao_3d_2014] identified in human lymphoblastoid cell line GM12878.
Chromatin loops are a level of 3D genome organization, formed when genomic loci (anchors) distant along the DNA double strand are in spatial proximity.
They have an important role in regulation of gene expression, bringing regulatory elements into contact with their target genes.
Several architectural proteins are involved in the formation of chromatin loops (e.g. CTCF in humans).
Besides acting as architectural/regulatory regions, many loop anchors encompass Transcription Start Sites of activated or repressed genes.
The existence of chromatin loops can be experimentally probed by chromatin conformation capture techniques, such as Hi-C, allowing to identify the loops as peaks in Hi-C contact maps.

We can inspect a few rows of the dataset:

```{r dataset}
data("genePairsGM12878")
head(genePairsGM12878)
```

The column names ending with `1` and `2` refer to the first and second loop anchors, respectively.
The essential columns are: `geneID1` and `geneID2` (gene identifiers from the Entrez database) and `interactionID` (identifier of a chromatin loop).
See the section [Extracting gene pairs from paired genomic regions] for how these gene pairs were obtained.

Of the 9,448 chromatin loops in the original study, 1,581 overlapped at least one gene Transcription Start Site (TSS) at both loop anchors.
As some loop anchors overlapped multiple TSSes, possibly of different genes, the dataset contains all combinations for these loops, yielding a total of 2,339 gene pairs:

```{r nrow}
nrow(genePairsGM12878)
```

From the first few rows you might have noticed that some gene pairs in this dataset are duplicated.
The approach used by GO-a-GO is to take each gene pair once only, i.e. to treat gene pairs as a set, similar to taking the set of genes in existing GO enrichment approaches.
Note that gene pair (A, B) is a duplicate of (B, A).
Also, gene pairs (A, A) containing the same gene twice are excluded by GO-a-GO as non-informative.
In this dataset, these cases correspond to chromatin loops between different TSSes of the same gene.
We will later see that the dataset contains 1,743 gene pairs that are unique and do not contain the same gene twice.

## Enrichment analysis of the example dataset

We will identify GO terms enriched in gene pairs associated with chromatin loops identified in human cell line GM12878.
The main function `GOaGO` takes as its first argument a data frame with columns `geneID1` and `geneID2` containing gene identifiers.
By setting `keyType`, we specify that the gene identifiers are from the Entrez database.
We will further use the Bioconductor `r Biocpkg("org.Hs.eg.db")` package as the source of Gene Ontology annotations for human genes, taking all three subontologies (Biological Process, Molecular Function, and Cellular Component) by setting `ont = "ALL"`.
We will run the default 10,000 permutations, and also use the default *p*-value cutoff of 0.05 with Benjamini-Hochberg correction:

```{r goago, message = FALSE}
library(org.Hs.eg.db)

# set the number of CPU threads to use
library(BiocParallel)
options(MulticoreParam = MulticoreParam(workers = 2))

goago <- GOaGO(genePairsGM12878,
    keyType = "ENTREZID", OrgDb = org.Hs.eg.db, ont = "ALL"
)
```

Note that we got warned about duplicated gene pairs and gene pairs containing the same gene twice, as discussed above.
We can now inspect the enriched GO terms, treating the resulting object of class `GOaGO-result` as a data frame:

```{r result}
head(as.data.frame(goago))
```

The enriched terms are internally ordered by *p*-value.
While this allows for an easy cut-off, it is not a good criterion alone.
We should further focus on the terms that have the highest `FoldEnrichment`, and also have sufficient `Count` (number of input gene pairs sharing the given term).

Note that some of the *p*-values have the value of zero, which means that for all the randomizations fewer gene pairs were associated with a given term than in the input dataset.
These values should be considered as less than 0.0001.

## Plotting the enriched Gene Ontology terms

We can plot the most enriched GO terms as a dotplot.
The X-axis shows the fold enrichment of the fraction of gene pairs sharing the given term, calculated as a quotient of `PairRatio` (the fraction in the input gene pairs) and `BgRatio` (the fraction in permuted ones).
Point size indicates the number of input gene pairs sharing the given term, and point color -- the adjusted *p*-value:

```{r dotplot, fig.cap = "Dotplot of 10 most enriched Gene Ontology terms"}
DotPlot(goago)
```

Note that by default only the terms associated with at least 5 gene pairs are shown; you can change this by setting `minCount` to any other value.
Furthermore, only 10 most enriched terms (with the highest `FoldEnrichment`) are plotted by default.
This can be changed by setting `showCategory` accordingly, possibly `showCategory = Inf` to plot all.

We can also see the sampling distributions of numbers of gene pairs sharing each GO term, obtained for the randomized gene pairs. From these distributions, empirical *p*-values were calculated:

```{r ridgeplot, fig.cap = "Ridgeplot of 10 most enriched Gene Ontology terms"}
RidgePlot(goago)
```

Given the shape of the above distributions, the low *p*-values are understandable.

## Comparison to unpaired Gene Ontology enrichment

GO-a-GO identifies terms that get preferentially paired in a set of gene pairs, addressing a complementary problem to the identification of enriched terms in a set of genes.
To confirm that the two approaches indeed yield different results, we will perform a comparison to GO term enrichment of the genes at loop anchors, without using the gene pair information.
For this unpaired approach, we will use `enrichGO` function from the widely used `r Biocpkg("clusterProfiler")` package [@yu_clusterprofiler_2012].

First, load the required packages:

```{r setup-comparison, message = FALSE}
library(clusterProfiler)
library(ggplot2)
library(ggrepel)
```

To extract the set of genes, we will go back to the example dataset of gene pairs.
Note that for a fair comparison we will exclude gene pairs (A, A), as these pairs are ignored by GO-a-GO.

```{r ego}
genes <- with(
    subset(genePairsGM12878, geneID1 != geneID2),
    unique(c(geneID1, geneID2))
)

ego <- enrichGO(genes,
    keyType = "ENTREZID", OrgDb = org.Hs.eg.db, ont = "ALL"
)
```

We can plot the most enriched GO terms identified by `enrichGO` as a dotplot:

```{r dotplot-ego, fig.cap = "Dotplot of 10 most enriched Gene Ontology terms, using unpaired approach by enrichGO"}
DotPlot(ego)
```

At first glance, the most enriched terms in this unpaired approach are very different from the ones identified by GO-a-GO.
This impression could be misleading, since we are plotting only the top 10 terms.
For a more detailed comparison, we will need more permissive (relaxed) versions of both enrichment analyses.

```{r goago-ego-relaxed}
goago_relaxed <- GOaGO(genePairsGM12878,
    keyType = "ENTREZID", OrgDb = org.Hs.eg.db, ont = "ALL",
    pvalueCutoff = Inf, qvalueCutoff = Inf
)

ego_relaxed <- enrichGO(genes,
    keyType = "ENTREZID", OrgDb = org.Hs.eg.db, ont = "ALL",
    pvalueCutoff = Inf, qvalueCutoff = Inf
)
```

In these two objects we have calculated statistics for `r format(nrow(as.data.frame(goago_relaxed)), big.mark = ",")` Gene Ontology terms that are shared by at least one gene pair, and separately for `r format(nrow(as.data.frame(ego_relaxed)), big.mark = ",")` terms that are associated to at least one gene.
We will merge them and flag the enriched terms according to the previous results.

```{r merge-go-ego}
df <- merge(as.data.frame(goago_relaxed), as.data.frame(ego_relaxed),
    by = c("ONTOLOGY", "ID", "Description"), suffixes = c(".GOaGO", ".enrichGO")
)

df$signif.GOaGO <- df$ID %in% as.data.frame(goago)$ID
df$signif.enrichGO <- df$ID %in% ego$ID

df$enrichment <- ifelse(df$signif.GOaGO,
    ifelse(df$signif.enrichGO, "goago_and_ego", "goago_only"),
    ifelse(df$signif.enrichGO, "ego_only", "insignificant")
)
```

We can now compare the fold enrichment calculated using the two approaches.
In the plot below, point size indicates the number of input gene pairs sharing the given term, and point color -- enrichment status.
For better visibility, we use log scale for both axes.

```{r comparison, fig.cap = "Gene Ontology term enrichment using unpaired approach by enrichGO (X-axis) and paired approach by GO-a-GO (Y-axis)", fig.wide = TRUE, fig.asp = 0.7}
col_labels <- c(
    goago_and_ego = "GO-a-GO and enrichGO",
    goago_only = "GO-a-GO (paired) only",
    ego_only = "enrichGO (unpaired) only",
    insignificant = "none of the above"
)

col_values <- c(
    goago_and_ego = "#6a3d9a",
    goago_only = "#e31a1c",
    ego_only = "#1f78b4",
    insignificant = "gray80"
)

ggplot(
    df,
    aes(x = log2(FoldEnrichment.enrichGO), y = log2(FoldEnrichment.GOaGO), size = Count.GOaGO, color = enrichment)
) +
    geom_vline(xintercept = 0, lty = 2) +
    geom_hline(yintercept = 0, lty = 2) +
    geom_point(alpha = 0.3) +
    scale_size_area() +
    scale_color_manual("Term enrichment", labels = col_labels, values = col_values) +
    DOSE::theme_dose(12)
```

You may be surprised by the fact that the vast majority of the plotted terms were called enriched by GO-a-GO.
This is due to the fact that `goago_relaxed` contains only the terms that are shared by at least one gene pair.

To get a better view of the most frequent annotations, we will plot only the terms associated with at least 5 gene pairs, and show term descriptions as point labels.
To repel overlapping labels, we use `geom_text_repel` from `r Biocpkg("ggrepel")` package, which also draws segments between points and their labels located further away.

```{r comparison-with-labels, fig.cap = "Gene Ontology term enrichment using unpaired approach by enrichGO (X-axis) and paired approach by GO-a-GO (Y-axis), showing only the terms shared by at least 5 gene pairs", fig.wide = TRUE, fig.asp = 0.7}
ggplot(
    subset(df, Count.GOaGO >= 5),
    aes(x = log2(FoldEnrichment.enrichGO), y = log2(FoldEnrichment.GOaGO), size = Count.GOaGO, color = enrichment)
) +
    geom_vline(xintercept = 0, lty = 2) +
    geom_point(alpha = 0.3) +
    geom_text_repel(aes(label = label_wrap_gen(50)(Description)), show.legend = FALSE, size = 3, lineheight = 0.9) +
    scale_size_area() +
    scale_color_manual("Term enrichment", labels = col_labels, values = col_values) +
    DOSE::theme_dose(12)
```

Many of the terms enriched in paired approach by GO-a-GO were not enriched in unpaired approach by enrichGO.
What is more, some of the terms reported as enriched by GO-a-GO are actually *depleted* when annotating the gene set without the information on gene pairs.
Explaining this conundrum requires further analysis and biological interpretation.

## Extracting gene pairs associated with terms of interest

For follow-up analysis, it is often desired to extract the enriched Gene Ontology terms along with the gene pairs sharing them.
Function `termGenePairs` returns a data frame including all this information, including gene symbols fetched from the `OrgDb` object.
For example, we can list the gene pairs associated with olfactory receptor activity:

```{r termGenePairs}
tgp <- termGenePairs(goago, org.Hs.eg.db)
subset(tgp, ID == "GO:0004984") # olfactory receptor activity
```

As can be seen in the last two columns, these gene pairs involve several genes from the olfactory receptor gene family, which, incidentally, is the largest gene family in the human genome.
Olfactory receptor genes are located in clusters at multiple loci distributed across 18 human chromosomes.
In olfactory sensory neurons, these gene clusters are brought together to form specific inter-chromosomal contacts [@monahan_lhx2-_2019].
The enrichment observed above suggests that some contacts within olfactory receptor gene clusters may have a role in non-olfactory tissues.

## Obtaining additional statistics

We can calculate the number of chromatin loops from the original study that overlapped at least one gene TSS at both loop anchors (should be 1,581):

```{r num_loops_with_genes}
length(unique(genePairsGM12878$interactionID))
```

and the number of input gene pairs that are unique and do not contain the same gene twice (should be 1,743):

```{r num_unique_gene_pairs}
nrow(genePairs(goago))
```

Further information is also reported by the method `show`, implicitly used here:

```{r show}
goago
```


# Extracting gene pairs from paired genomic regions

We started this vignette with an example dataset of gene pairs.
These pairs are often defined by a set of chromatin interactions, such as chromatin loops (as it is the case here), differential contacts or spatial gene clusters.
What these scenarios have in common is that the underlying genomic regions need to be annotated to obtain gene pairs.

We will use the Bioconductor package `r Biocpkg("rtracklayer")` to read a file in [BEDPE format](https://bedtools.readthedocs.io/en/latest/content/general-usage.html#bedpe-format) containing genomic coordinates of the 9,448 chromatin loops discussed before:

```{r import-bedpe, message = FALSE}
fpath <- system.file("extdata", "GM12878_loops.bedpe.gz", package = "GOaGO")
pairs <- rtracklayer::import(fpath, genome = "hg19")
pairs
```

The `first` and `second` genomic regions shown above correspond to the first and second loop anchors.
These paired regions will be further associated to relevant genes.
We will use transcript database from the annotation package `r Biocpkg("TxDb.Hsapiens.UCSC.hg19.knownGene")`.
For more informative associations we will take only the transcripts of protein-coding genes.
This can be done by applying a filter that ensures that the coding DNA sequence (CDS) strand is `"-"` or `"+"`, i.e. not `NA`:

```{r transcripts, message = FALSE}
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
transcripts <- transcripts(TxDb.Hsapiens.UCSC.hg19.knownGene,
    columns = "gene_id", filter = list(cds_strand = c("-", "+"))
)
```

From these transcripts, only the Transcription Start Site (TSS) position will be used for annotation.
Each genomic region (loop anchor) will be associated to all the TSSes it overlaps.
If there is no such overlap, the region will be associated to the closest TSS within 10 kb distance, if there are any.
As some regions overlapped multiple TSSes, possibly of different genes, the resulting annotation contains all the combinations:

```{r annotateInteractions}
genePairs <- annotateInteractions(pairs, transcripts, maxDistanceToTSS = 10e3)
genePairs
```

We obtain the same result as provided in the dataset `genePairsGM12878`.
Note that the information on `keyType` is extracted from `transcripts` and propagated to gene pairs:

```{r attr-keyType}
attr(genePairs, "keyType")
```

so that we do not need to specify `keyType` when running `GOaGO` function on this object.

For further manipulation of paired genomic regions, and specifically chromatin contacts, Bioconductor packages `r Biocpkg("GenomicInteractions")` and `r Biocpkg("mariner")` can be useful.
In particular, the function `annotateInteractions` can take objects of class `GenomicInteractions`.


# Citing GO-a-GO

We hope that GO-a-GO will be useful for your research. Please use the following information to cite the package and the overall approach.

```{r citation}
citation("GOaGO")
```


# Session information

```{r sessionInfo, echo = FALSE}
sessionInfo()
```


# Bibliography

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R, R/methods-show.R
\name{GOaGO-accessors}
\alias{GOaGO-accessors}
\alias{genePairs}
\alias{keyType}
\alias{organism}
\alias{organism,GOaGO-result-method}
\alias{show}
\alias{show,GOaGO-result-method}
\title{Accessors and show method for \code{GOaGO-result} objects}
\usage{
genePairs(object)

keyType(object)

organism(object)

show(object)
}
\arguments{
\item{object}{of class \code{GOaGO-result}}
}
\value{
\code{genePairs} returns a data frame with the input gene pairs, with the
columns \code{geneID1}, \code{geneID2} and \code{pairID}.

\code{keyType} returns the type of gene identifiers, such as "ENTREZID" or
"ENSEMBL".

\code{organism} returns the scientific name (i.e. genus and species, or
genus and species and subspecies) of the organism.

\code{show} displays the object, and returns an invisible \code{NULL}.
}
\description{
Accessors and show method for \code{GOaGO-result} objects
}
\examples{
library(org.Hs.eg.db)
data("genePairsGM12878")

goago <- GOaGO(genePairsGM12878, keyType = "ENTREZID", OrgDb = org.Hs.eg.db)
show(goago)

genePairs(goago)
keyType(goago)
organism(goago)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOaGO.R
\name{GOaGO}
\alias{GOaGO}
\title{Gene Ontology enrichment analysis in a set of gene pairs}
\usage{
GOaGO(
  genePairs,
  OrgDb,
  keyType = NULL,
  ont = "MF",
  minCount = 1,
  numPermutations = 10000,
  universe,
  pvalueCutoff = 0.05,
  pAdjustMethod = "BH",
  qvalueCutoff = 0.2,
  minGSSize = 10,
  maxGSSize = 500
)
}
\arguments{
\item{genePairs}{a data frame with columns \code{geneID1} and \code{geneID2}
containing gene identifiers; column \code{pairID} will also be used if
provided.}

\item{OrgDb}{OrgDb}

\item{keyType}{type of gene identifiers, such as "ENTREZID" or "ENSEMBL", if
it cannot be determined from metadata of \code{genePairs}}

\item{ont}{one of "BP", "MF", and "CC" subontologies, or "ALL" for all three}

\item{minCount}{cutoff for number of pairs that share a GO term for this
term to be considered}

\item{numPermutations}{number of permutations performed in the enrichment
test}

\item{universe}{a set of background genes. If missing, all the genes from
all the gene pairs will be used as background.}

\item{pvalueCutoff}{adjusted p-value cutoff on enrichment tests to report}

\item{pAdjustMethod}{one of "holm", "hochberg", "hommel", "bonferroni",
"BH", "BY", "fdr", "none"}

\item{qvalueCutoff}{q-value cutoff on enrichment tests to report as
significant. Tests must pass (i) \code{pvalueCutoff} on unadjusted
p-values, (ii) \code{pvalueCutoff} on adjusted p-values, and (iii)
\code{qvalueCutoff} on q-values to be reported.}

\item{minGSSize}{minimal size of genes annotated for testing}

\item{maxGSSize}{maximal size of genes annotated for testing}
}
\value{
A \code{GOaGO-result} object.
}
\description{
Given a data frame of gene pairs, this function will return the enriched
Gene Ontology terms after FDR control.
}
\examples{
library(org.Hs.eg.db)
data("genePairsGM12878")

goago <- GOaGO(genePairsGM12878, keyType = "ENTREZID", OrgDb = org.Hs.eg.db)
show(goago)
}
\seealso{
\code{\link{GOaGO-result-class}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotateInteractions.R
\name{annotateInteractions}
\alias{annotateInteractions}
\title{Associate interactions to gene pairs}
\usage{
annotateInteractions(
  interactions,
  transcripts = NULL,
  tss = NULL,
  keyType = NULL,
  maxDistanceToTSS = 0
)
}
\arguments{
\item{interactions}{object of class \code{Pairs} (of \code{GRanges}) or
\code{GenomicInteractions}}

\item{transcripts}{\code{TxDb} annotation or other \code{GRanges} object}

\item{tss}{object of class \code{GRanges} as returned by
\code{\link{convertTranscriptsToTSS}}}

\item{keyType}{type of gene identifiers, such as "ENTREZID" or "ENSEMBL", if
it cannot be determined from metadata of \code{transcripts} or \code{tss}}

\item{maxDistanceToTSS}{maximal distance to extend the search for nearest
TSS outside the anchor}
}
\value{
A data table with columns \code{interactionID} (index of the
  interaction), \code{chrom1}, \code{start1}, \code{end1}, \code{chrom2},
  \code{start2}, \code{end2} (coordinates of both anchors), \code{geneID1},
  \code{geneID2} (gene identifiers from `transcripts` or `tss` for both
  anchors), \code{tss1}, \code{tss2} (TSS position for both anchors),
  \code{strand1} and \code{strand2} (TSS strand for both anchors).
}
\description{
Both interaction anchors are associated to TSSes as described in
\code{\link{annotateAnchors}}. Briefly, each anchor is associated to all the
TSSes it overlaps, or to all closest TSSes up to \code{maxDistanceToTSS} if
there is no such overlap. The annotation of an interaction is a Cartesian
product of annotations for both anchors.
}
\details{
Either \code{transcripts} or \code{tss} must be provided, but not both.
}
\examples{
library(TxDb.Hsapiens.UCSC.hg19.knownGene)

# take only the transcripts of coding genes by ensuring that the coding
# sequence strand is not NA
transcripts <- transcripts(TxDb.Hsapiens.UCSC.hg19.knownGene,
    columns = "gene_id", filter = list(cds_strand = c("-", "+"))
)

fpath <- system.file("extdata", "GM12878_loops.bedpe.gz", package = "GOaGO")
pairs <- rtracklayer::import(fpath, genome = "hg19")
annotateInteractions(pairs, transcripts, maxDistanceToTSS = 10e3)
}
\seealso{
\code{\link{convertTranscriptsToTSS}},
  \code{\link{annotateAnchors}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotateInteractions.R
\name{convertTranscriptsToTSS}
\alias{convertTranscriptsToTSS}
\title{Convert gene transcripts to Transcription Start Sites}
\usage{
convertTranscriptsToTSS(
  transcripts,
  geneid_column = c("gene_id", "GENEID", "geneID")
)
}
\arguments{
\item{transcripts}{\code{TxDb} annotation or other \code{GRanges} object}

\item{geneid_column}{A character vector of recognized names for the metadata
column in \code{transcripts} that contains gene identifiers. If none or more
than one is found, an error is raised.}
}
\value{
A \code{GRanges} object with gene identifiers in metadata column
\code{geneID} being a vector.
}
\description{
Convert a \code{GRanges} object with gene transcripts to a \code{GRanges}
object with gene TSSes of length 1 bp, with duplicate rows removed. Assumes
that gene identifiers are provided in one of the metadata columns, either as
a vector (possibly containing \code{NA} values) or a \code{CharacterList}.
The function is idempotent, i.e. can be applied multiple times without
changing the result.
}
\examples{
library(TxDb.Hsapiens.UCSC.hg19.knownGene)

# take only the transcripts of coding genes by ensuring that the coding
# sequence strand is not NA
transcripts <- transcripts(TxDb.Hsapiens.UCSC.hg19.knownGene,
    columns = "gene_id", filter = list(cds_strand = c("-", "+"))
)

convertTranscriptsToTSS(transcripts)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOaGO.R
\docType{class}
\name{GOaGO-result-class}
\alias{GOaGO-result-class}
\title{An S4 class to represent the results of GO-a-GO enrichment analysis}
\description{
An S4 class to represent the results of GO-a-GO enrichment analysis
}
\section{Slots}{

\describe{
\item{\code{result}}{A data frame of the enriched Gene Ontology terms, with the
following columns: \code{ONTOLOGY}, \code{ID}, \code{Description} (all of
the GO term), \code{Count} (number of input gene pairs sharing the given
term), \code{PairRatio} (fraction of input gene pairs sharing the given
term), \code{BgRatio} (fraction of permuted gene pairs sharing the given
term), \code{FoldEnrichment} (quotient of the two fractions),
\code{pvalue}, \code{p.adjust}, \code{qvalue}.}

\item{\code{pvalueCutoff}}{adjusted p-value cutoff on enrichment tests}

\item{\code{pAdjustMethod}}{p-value adjustment method}

\item{\code{qvalueCutoff}}{q-value cutoff on enrichment tests}

\item{\code{minCount}}{cutoff for number of pairs that share a GO term for this term
to be considered}

\item{\code{numPermutations}}{number of permutations performed in the enrichment
test}

\item{\code{minGSSize}}{minimal size of genes annotated for testing}

\item{\code{maxGSSize}}{maximal size of genes annotated for testing}

\item{\code{organism}}{scientific name of the organism}

\item{\code{ontology}}{one of "BP", "MF", and "CC" subontologies, or "ALL" for all
three}

\item{\code{keyType}}{type of gene identifiers, such as "ENTREZID" or "ENSEMBL"}

\item{\code{genePairs}}{A data frame with the input gene pairs, with the columns
\code{geneID1}, \code{geneID2} and \code{pairID}.}

\item{\code{pairTerms}}{A data frame linking the enriched Gene Ontology terms with
the input gene pairs, with the columns \code{pairID} and \code{ID} (of the
GO term).}

\item{\code{permutedResult}}{A data frame with the columns \code{ID} (of the GO
term) and \code{Count}, keeping the numbers of permuted gene pairs sharing
the term as obtained in every random permutation.}

\item{\code{universe}}{a set of background genes}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{DotPlot}
\alias{DotPlot}
\title{Dotplot of the enriched Gene Ontology terms}
\usage{
DotPlot(
  object,
  minCount = 5,
  x = "FoldEnrichment",
  color = "p.adjust",
  size = "Count",
  showCategory = 10,
  orderBy = "FoldEnrichment",
  decreasing = TRUE,
  font.size = 12,
  label_format = 50
)
}
\arguments{
\item{object}{GO-a-GO results of class \code{GOaGO-result}}

\item{minCount}{plot only the GO terms that are associated to at least the
given number of gene pairs}

\item{x}{Variable for X-axis, one of \code{"FoldEnrichment"},
\code{"PairRatio"} and \code{"Count"}.}

\item{color}{Variable used to color enriched terms, e.g. \code{"pvalue"},
\code{"p.adjust"} or \code{"qvalue"}.}

\item{size}{Variable used to scale the sizes of points, one of
\code{"FoldEnrichment"}, \code{"PairRatio"} and \code{"Count"}.}

\item{showCategory}{number of terms to display or a vector of terms}

\item{orderBy}{The order of the Y-axis, one of \code{"FoldEnrichment"},
\code{"PairRatio"} and \code{"Count"}.}

\item{decreasing}{logical. Should the \code{orderBy} order be increasing or
decreasing?}

\item{font.size}{font size}

\item{label_format}{a numeric value sets wrap length, alternatively a custom
function to format axis labels. By default wraps names longer than 50
characters.}
}
\value{
A \code{ggplot} object that can be further customized using the
  \code{ggplot2} package.
}
\description{
By default, plots the most enriched terms, with fold enrichment on the
X-axis, point size indicating the number of gene pairs sharing the given
term, and point color -- the adjusted p-value.
}
\examples{
library(org.Hs.eg.db)
data("genePairsGM12878")

goago <- GOaGO(genePairsGM12878, keyType = "ENTREZID", OrgDb = org.Hs.eg.db)
DotPlot(goago)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOaGO-package.R
\docType{package}
\name{GOaGO-package}
\alias{GOaGO-package}
\title{GOaGO: Gene Ontology enrichment analysis of gene pairs}
\description{
GO-a-GO annotates Gene Ontology terms that are enriched in a given set of gene pairs. The enrichment is calculated from a permutation test for overrepresentation of gene pairs that are associated with a shared term. Such gene pairs are counted for the original set of gene pairs and compared against randomized sets in which the structure of the pairs is preserved, but the gene identities (including the associated terms) are permuted.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/ajank/GOaGO}
  \item Report bugs at \url{https://github.com/ajank/GOaGO/issues}
}

}
\author{
\strong{Maintainer}: Aleksander Jankowski \email{aleksander.jankowski@uw.edu.pl} (\href{https://orcid.org/0000-0002-2212-6224}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOaGO.R
\name{uniqueGenePairs}
\alias{uniqueGenePairs}
\title{Extract unique gene pairs from the data frame provided}
\usage{
uniqueGenePairs(genePairs)
}
\arguments{
\item{genePairs}{a data frame with columns \code{geneID1} and \code{geneID2}
containing gene identifiers; column \code{pairID} will also be used if
provided.}
}
\value{
A data frame with columns \code{pairID}, \code{geneID1} and
  \code{geneID2}. If loops or duplicates were removed, a warning will
  alert you. If column \code{pairID} was not provided in \code{genePairs},
  an integer vector equal to \code{seq_len(nrow(result))} will be used.
}
\description{
Given a data frame of gene pairs, this function will return the unique
pairs of genes, removing loops (gene pairs containing the same gene twice)
and duplicates. Note that gene pair (A, B) is a duplicate of (B, A).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\docType{methods}
\name{as.data.frame}
\alias{as.data.frame}
\alias{as.data.frame,GOaGO-result-method}
\title{Coerce a \code{GOaGO-result} object to a data frame}
\usage{
as.data.frame(x, row.names=NULL, optional=FALSE, ...)
}
\arguments{
\item{x}{The object to coerce.}

\item{row.names, optional, ...}{Not used, inherited from
\code{base::as.data.frame()}.}
}
\value{
A data frame of the enriched Gene Ontology terms, with the
  following columns: \code{ONTOLOGY}, \code{ID}, \code{Description} (all of
  the GO term), \code{Count} (number of input gene pairs sharing the given
  term), \code{PairRatio} (fraction of input gene pairs sharing the given
  term), \code{BgRatio} (fraction of permuted gene pairs sharing the given
  term), \code{FoldEnrichment} (quotient of the two fractions),
  \code{pvalue}, \code{p.adjust}, \code{qvalue}.
}
\description{
Coerce a \code{GOaGO-result} object to a data frame
}
\examples{
library(org.Hs.eg.db)
data("genePairsGM12878")

goago <- GOaGO(genePairsGM12878, keyType = "ENTREZID", OrgDb = org.Hs.eg.db)
as.data.frame(goago)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/termGenePairs.R
\name{termGenePairs}
\alias{termGenePairs}
\title{Extract the enriched Gene Ontology terms along with gene pairs sharing them}
\usage{
termGenePairs(object, OrgDb = NULL)
}
\arguments{
\item{object}{of class \code{GOaGO-result}}

\item{OrgDb}{OrgDb to map gene identifiers to gene symbols}
}
\value{
A data frame similar to returned by \code{as.data.frame(object)},
  but including all the gene pairs sharing each enriched Gene Ontology term,
  one gene pair in each row. Additional columns include \code{pairID},
  \code{geneID1}, \code{geneID2} and any other columns provided in
  \code{genePairs} argument to \code{\link{GOaGO}}. If \code{OrgDb} is
  provided, \code{geneSymbol1} and \code{geneSymbol2} will also be added.
}
\description{
Extract the enriched Gene Ontology terms along with gene pairs sharing them
}
\examples{
library(org.Hs.eg.db)
data("genePairsGM12878")

goago <- GOaGO(genePairsGM12878, keyType = "ENTREZID", OrgDb = org.Hs.eg.db)
termGenePairs(goago, OrgDb = org.Hs.eg.db)
}
\seealso{
\code{\link{as.data.frame,GOaGO-result-method}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotateInteractions.R
\name{annotateAnchors}
\alias{annotateAnchors}
\title{Associate interaction anchors to the nearest TSSes}
\usage{
annotateAnchors(
  anchors,
  transcripts = NULL,
  tss = NULL,
  keyType = NULL,
  maxDistanceToTSS = 0
)
}
\arguments{
\item{anchors}{object of class \code{GRanges}}

\item{transcripts}{\code{TxDb} annotation or other \code{GRanges} object}

\item{tss}{object of class \code{GRanges} as returned by
\code{\link{convertTranscriptsToTSS}}}

\item{keyType}{type of gene identifiers, such as "ENTREZID" or "ENSEMBL", if
it cannot be determined from metadata of \code{transcripts} or \code{tss}}

\item{maxDistanceToTSS}{maximal distance to extend the search for nearest
TSS outside the anchor}
}
\value{
A data table with columns \code{interactionID} (index of the
  anchor), \code{chrom}, \code{start}, \code{end} (coordinates of the
  anchor), \code{geneID} (gene identifier from `transcripts` or `tss`),
  \code{tss} (TSS position) and \code{strand} (TSS strand).
}
\description{
Interaction anchors are associated to TSSes as follows. Each anchor is
associated to all the TSSes it overlaps. If there is no such overlap, then
the anchor is associated to all TSSes with the shortest distance to the
anchor, if this distance is not larger than \code{maxDistanceToTSS}.
}
\details{
Either \code{transcripts} or \code{tss} must be provided, but not both.
}
\examples{
library(TxDb.Hsapiens.UCSC.hg19.knownGene)

# take only the transcripts of coding genes by ensuring that the coding
# sequence strand is not NA
transcripts <- transcripts(TxDb.Hsapiens.UCSC.hg19.knownGene,
    columns = "gene_id", filter = list(cds_strand = c("-", "+"))
)

tss <- convertTranscriptsToTSS(transcripts)
gr <- GRanges("chr1", IRanges(c(42001, 890001), c(62000, 900000)))

# note that anchors are associated to TSSes outside the anchor only if there
# are no TSSes overlapping the anchor
annotateAnchors(gr, tss, maxDistanceToTSS = 10e3)

# this may yield more associations to TSSes outside the anchors
annotateAnchors(gr + 10e3, tss)
}
\seealso{
\code{\link{convertTranscriptsToTSS}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{genePairsGM12878}
\alias{genePairsGM12878}
\title{Gene pairs associated with chromatin loops in GM12878 cell line}
\format{
A data frame with 2,339 rows and 13 columns:
\describe{
  \item{interactionID}{loop identifier}
  \item{chrom1}{chromosome of loop anchor 1}
  \item{start1}{start coordinate of loop anchor 1}
  \item{end1}{end coordinate of loop anchor 1}
  \item{geneID1}{Entrez identifier of the gene associated to loop anchor 1}
  \item{tss1}{TSS coordinate of the associated gene}
  \item{strand1}{strand (\code{"+"} or \code{"-"}) of the associated gene}
  \item{chrom2}{chromosome of loop anchor 2}
  \item{start2}{start coordinate of loop anchor 2}
  \item{end2}{end coordinate of loop anchor 2}
  \item{geneID2}{Entrez identifier of the gene associated to loop anchor 2}
  \item{tss2}{TSS coordinate of the associated gene}
  \item{strand2}{strand (\code{"+"} or \code{"-"}) of the associated gene.}
}
}
\source{
https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE63525
}
\usage{
genePairsGM12878
}
\description{
The dataset is based on 9,448 chromatin loops identified in human cell line
GM12878 as peaks in Hi-C contact maps. Of these chromatin loops, 1,581
overlapped at least one gene Transcription Start Site (TSS) at both loop
anchors. As some loop anchors overlapped multiple TSSes, possibly of
different genes, the dataset contains all combinations for these loops,
yielding a total of 2,339 gene pairs, of which 1,743 pairs are unique and do
not contain the same gene twice.
}
\references{
Rao, S. S., Huntley, M. H., \emph{et al.} (2014). A 3D map of
  the human genome at kilobase resolution reveals principles of chromatin
  looping. \emph{Cell}, 159(7), 1665-80.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{RidgePlot}
\alias{RidgePlot}
\title{Ridgeplot of the sampling distributions for the randomized gene pairs}
\usage{
RidgePlot(
  object,
  minCount = 5,
  showCategory = 10,
  orderBy = "FoldEnrichment",
  decreasing = TRUE,
  font.size = 12,
  label_format = 50
)
}
\arguments{
\item{object}{GO-a-GO results of class \code{GOaGO-result}}

\item{minCount}{plot only the GO terms that are associated to at least the
given number of gene pairs}

\item{showCategory}{number of terms to display or a vector of terms}

\item{orderBy}{The order of the Y-axis, one of \code{"FoldEnrichment"},
\code{"PairRatio"} and \code{"Count"}.}

\item{decreasing}{logical. Should the \code{orderBy} order be increasing or
decreasing?}

\item{font.size}{font size}

\item{label_format}{a numeric value sets wrap length, alternatively a custom
function to format axis labels. By default wraps names longer than 50
characters.}
}
\value{
A \code{ggplot} object that can be further customized using the
  \code{ggplot2} package.
}
\description{
Ridgeplot of the sampling distributions of numbers of gene pairs sharing
each enriched Gene Ontology term, obtained for the randomized gene pairs.
}
\examples{
library(org.Hs.eg.db)
data("genePairsGM12878")

goago <- GOaGO(genePairsGM12878, keyType = "ENTREZID", OrgDb = org.Hs.eg.db)
RidgePlot(goago)
}
